import matplotlib
import matplotlib.pyplot as plt
import matplotlib as mpl
import meteva
import numpy as np
import  math
from  matplotlib import  cm
from matplotlib.ticker import NullFormatter, FixedLocator
import copy
import pandas as pd
import datetime
import scipy.stats as st


def scatter_regress(ob, fo,member_list = None, rtype="linear",vmax = None,vmin = None, ncol = None,save_path=None,show = False,dpi = 300, title="散点回归图",
                    sup_fontsize = 10,width = None,height = None,ylabel ="观测"):
    '''
    绘制观测-预报散点图和线性回归曲线
    :param Ob: 实况数据  任意维numpy数组
    :param Fo: 预测数据 任意维numpy数组,Fo.shape 和Ob.shape一致
    :param save_path:图片保存路径，缺省时不输出图片，而是以默认绘图窗口形式展示
    :return:图片，包含散点图和线性回归图,横坐标为观测值，纵坐标为预报值，横坐标很纵轴标取值范围自动设为一致，在图形中间添加了完美预报的参考线。
    '''

    num_max = max(np.max(ob), np.max(fo))
    num_min = min(np.min(ob), np.min(fo))
    dmm = num_max - num_min
    if (num_min < 0):
        num_min -= 0.1 * dmm
    else:
        num_min -= 0.1 * dmm
        if num_min < 0:  # 如果开始全大于，则最低值扩展不超过0
            num_min = 0
    num_max += dmm * 0.1
    if vmax is not None:
        num_max = vmax
    if vmin is not None:
        num_min = vmin
    dmm = num_max - num_min


    Fo_shape = fo.shape
    Ob_shape = ob.shape
    Ob_shpe_list = list(Ob_shape)
    size = len(Ob_shpe_list)
    ind = -size
    Fo_Ob_index = list(Fo_shape[ind:])
    if Fo_Ob_index != Ob_shpe_list:
        print('预报数据和观测数据维度不匹配')
        return
    Ob_shpe_list.insert(0, -1)
    new_Fo_shape = tuple(Ob_shpe_list)
    new_Fo = fo.reshape(new_Fo_shape)
    new_Fo_shape = new_Fo.shape
    sub_plot_num = new_Fo_shape[0]

    if ncol is None:
        if sub_plot_num ==1:
            ncols = 1
        elif sub_plot_num %2 == 0:
            ncols = 2
        else:
            ncols = 3
    else:
        ncols = ncol

    nrows = math.ceil(new_Fo_shape[0] / ncols)

    if height is None:
        if nrows==1:
            if ncols <3:
                height_fig = 3.5
            else:
                height_fig = 2.5
        else:
            if ncols > nrows:
                height_fig = 6
            else:
                height_fig = 7
    else:
        height_fig = height

    height_suptitle = 0.4
    height_xticks_title = 0.1
    height_hspace = 0.6
    heidht_axis = (height_fig - height_suptitle - height_xticks_title - height_hspace * (nrows - 1)) / nrows
    width_axis = heidht_axis
    width_yticks = 0.1
    width_wspace = width_yticks * 5
    if width is None:
        width_fig = width_axis * ncols + width_wspace * (ncols - 1) + width_yticks
    else:
        width_fig = width

    grid_count = meteva.base.grid([num_min,num_max,(num_max-num_min)/100],[num_min,num_max,(num_max-num_min)/100])


    fig = plt.figure(figsize=(width_fig,height_fig),dpi = dpi)
    for line in range(new_Fo_shape[0]):
        ob = ob.flatten()
        fo = new_Fo[line,:].flatten()
        markersize = width_axis * heidht_axis / np.sqrt(ob.size)
        if markersize < 0.3:
            markersize = 0.3
        elif markersize > 20:
            markersize = 20

        plt.subplot(nrows, ncols, line + 1)
        df =pd.DataFrame({"lon":fo,"lat":ob})
        sta_xy = meteva.base.sta_data(df)
        sta_xy["time"] = datetime.datetime(2020,1,1,0)
        sta_xy["data0"] = 1
        grd_count = meteva.base.near.add_stavalue_to_nearest_grid(sta_xy,grid = grid_count)
        sta_count = meteva.base.interp_gs_linear(grd_count,sta_xy)
        colors = sta_count["data0"]
        sort_index = colors.argsort()
        fo_s = fo[sort_index]
        ob_s = ob[sort_index]
        colors = colors[sort_index]
        #plt.scatter(fo_s, ob_s, c=colors,s = markersize,cmap="tab20c")

        try:
            plt.scatter(fo_s, ob_s, c=colors, s=markersize, cmap="turbo")
        except:
            plt.scatter(fo_s, ob_s, c=colors, s=markersize, cmap="rainbow")
        #plt.plot(fo, ob, '.', color='b', markersize=markersize)

        plt.subplots_adjust(left=0, bottom=0.0, right=1.0, top = 1 - height_suptitle/height_fig,
                            hspace=height_hspace/heidht_axis,wspace=width_wspace/width_axis)
        if rtype == "rate":
            ob_line = np.arange(num_min, num_max, dmm / 30)
            rate = np.mean(ob) / np.mean(fo)
            fo_rg = ob_line * np.mean(ob) / np.mean(fo)
            plt.plot(ob_line, fo_rg, color="k")
            rg_text2 = "Y = " + '%.2f' % rate + "X"
            plt.text(num_min + 0.05 * dmm, num_min + 0.92 * dmm, rg_text2, fontsize=0.8 * sup_fontsize, color="r")
        elif rtype == "linear":
            #X = np.zeros((len(fo), 1))
            #X[:, 0] = fo
            #clf = LinearRegression().fit(X, ob)
            # 斜率，截距，r 值，p 值，标准误差
            slope, intercept, r_value, p_value, std_err = st.linregress(fo, ob)
            ob_line = np.arange(num_min, num_max, dmm / 30)
            # X = np.zeros((len(ob_line), 1))
            # X[:, 0] = ob_line
            #fo_rg = clf.predict(X)
            fo_rg = slope * ob_line + intercept
            plt.plot(ob_line, fo_rg, color="k")
            #rg_text2 = "Y = " + '%.2f' % (clf.coef_[0]) + "* X + " + '%.2f' % (clf.intercept_)
            if intercept >=0:
                rg_text2 = "Y = " + '%.2f' % (slope) + "* X + " + '%.2f' % (intercept)
            else:
                rg_text2 = "Y = " + '%.2f' % (slope) + "* X - " + '%.2f' % (-intercept)

            plt.text(num_min + 0.05 * dmm, num_min + 0.92 * dmm, rg_text2, fontsize=0.8 * sup_fontsize, color="r")
        corr1 = meteva.method.corr(ob,fo)
        re_text1 = "corr = "+ '%.2f' % (corr1)
        plt.text(num_min + 0.05 * dmm, num_min + 0.85 * dmm, re_text1, fontsize=0.8 * sup_fontsize, color="r")


        plt.plot(ob_line, ob_line, '--', color="k",linewidth = 0.5)
        plt.xlim(num_min, num_max)
        plt.ylim(num_min, num_max)
        plt.xticks(fontsize = 0.8 * sup_fontsize)
        plt.yticks(fontsize = 0.8 * sup_fontsize)


        if member_list is None:
            #plt.title('预报'+str(line+1),fontsize = 0.9 * fontsize_sup)
            plt.xlabel('预报'+str(line+1), fontsize=0.9 * sup_fontsize)
        else:
            #plt.title(member_list[line],fontsize = 0.9 * fontsize_sup)
            plt.xlabel(member_list[line], fontsize=0.9 * sup_fontsize)
        #plt.xlabel("预报", fontsize=0.9 * fontsize_sup)

        plt.ylabel(ylabel, fontsize=0.9 * sup_fontsize)
        plt.rcParams['xtick.direction'] = 'in'  # 将x轴的刻度线方向设置抄向内
        plt.rcParams['ytick.direction'] = 'in'  # 将y轴的刻度方知向设置向内
        #plt.grid(linestyle='--', linewidth=0.5)
    titlelines = title.split("\n")
    fig.suptitle(title, fontsize=sup_fontsize, y=0.99+0.01 * len(titlelines))

    if save_path is None:
        show = True
    else:
        meteva.base.tool.path_tools.creat_path(save_path)
        plt.savefig(save_path,bbox_inches='tight')
        print("检验结果已以图片形式保存至" + save_path)
    if show:
        plt.show()
    plt.close()
    return None




def pdf_plot(ob, fo,member_list = None,vmax = None,vmin = None, save_path=None,  show = False,dpi = 300,title="频率匹配检验图",
             sup_fontsize = 10,width = None,height = None,yscale = None,grid = False,percent = [0,1]):
    '''
    sorted_ob_fo 将传入的两组数据先进行排序
    然后画出折线图
    ----------------
    :param Ob: 实况数据  任意维numpy数组
    :param Fo: 预测数据 任意维numpy数组,Fo.shape 和Ob.shape一致
    :param save_path: 图片保存路径，缺省时不输出图片，而是以默认绘图窗口形式展示
    :return:图片，包含频率匹配映射关系图,横坐标为观测值，纵坐标为预报值，横坐标很纵轴标取值范围自动设为一致，在图形中间添加了完美预报的参考线。
    '''
    Fo_shape = fo.shape
    Ob_shape = ob.shape
    Ob_shpe_list = list(Ob_shape)
    size = len(Ob_shpe_list)
    ind = -size
    Fo_Ob_index = list(Fo_shape[ind:])
    if Fo_Ob_index != Ob_shpe_list:
        print('预报数据和观测数据维度不匹配')
        return
    Ob_shpe_list.insert(0, -1)
    new_Fo_shape = tuple(Ob_shpe_list)
    new_Fo = fo.reshape(new_Fo_shape)
    new_Fo_shape = new_Fo.shape

    if width is None:
        width = 8
    if height is None:
        height = width * 0.45

    fig = plt.figure(figsize=(width, height),dpi = dpi)
    num_max = max(np.max(ob), np.max(fo))
    num_min = min(np.min(ob), np.min(fo))
    dmm = num_max - num_min
    if (num_min != 0):
        num_min -= 0.1 * dmm
    num_max += dmm * 0.1
    if vmax is not None:
        num_max = vmax
    if vmin is not None:
        num_min = vmin
    dmm = num_max - num_min
    ob= ob.flatten()
    start_index = int(ob.size * percent[0])
    end_index= int(ob.size * percent[1])

    ob_sorted = np.sort(ob.flatten())

    ob_sorted_smooth = ob_sorted
    ob_sorted_smooth[1:-1] = 0.5 * ob_sorted[1:-1] + 0.25 * (ob_sorted[0:-2] + ob_sorted[2:])
    ob_sorted_smooth = ob_sorted_smooth[start_index:end_index]
    ax = plt.subplot(1, 2, 1)
    y = np.arange(len(ob_sorted_smooth)) / (len(ob_sorted_smooth))
    plt.plot(ob_sorted_smooth, y, label="观测")

    dss = ob_sorted_smooth - ob_sorted_smooth[0]
    fnq =[(dss != 0).argmax()]

    for line in range(new_Fo_shape[0]):
        if member_list is None:
            if new_Fo_shape[0] == 1:
                label = '预报'
            else:
                label = '预报' + str(line + 1)
        else:
            label = member_list[line]
        fo_sorted = np.sort(new_Fo[line, :].flatten())
        fo_sorted_smooth = fo_sorted
        fo_sorted_smooth[1:-1] = 0.5 * fo_sorted[1:-1] + 0.25 * (fo_sorted[0:-2] + fo_sorted[2:])
        fo_sorted_smooth = fo_sorted_smooth[start_index:end_index]

        plt.plot(fo_sorted_smooth, y, label=label)
        plt.xlabel("变量值", fontsize=0.9 * sup_fontsize)
        #plt.xlim(num_min, num_max)

        plt.ylabel("累积概率", fontsize=0.9 * sup_fontsize)
        plt.title("概率分布函数对比图", fontsize=0.9 * sup_fontsize)
        yticks = np.arange(0, 1.01, 0.1)
        plt.yticks(yticks, fontsize=0.8 * sup_fontsize)
        plt.xticks(fontsize=0.8 * sup_fontsize)
        plt.legend(loc="lower right")
        if yscale =="log":
            ax.set_yscale('log')
            major_locator = [0.00001,0.0001,0.001,0.01,0.1,0.5]
            ax.yaxis.set_minor_formatter(NullFormatter())
            ax.yaxis.set_major_locator(FixedLocator(major_locator))
            ax.set_yticklabels(major_locator)
        elif yscale == "logit":
            major_locator=[0.1,0.5,0.9,0.99,0.999,0.9999,0.99999]
            ax.set_yscale('logit')
            ax.yaxis.set_minor_formatter(NullFormatter())
            ax.yaxis.set_major_locator(FixedLocator(major_locator))
            ax.set_yticklabels(major_locator)
            dss = fo_sorted_smooth - fo_sorted_smooth[0]
            fnq.append((dss!=0).argmax())

        else:
            plt.ylim(0, 1)

    if yscale == "logit":
        minfnq = min(fnq)/ob_sorted_smooth.size
        plt.ylim(minfnq,1)


        #plt.yscale('logit')
    if grid:
        plt.grid()

    plt.subplot(1, 2, 2)
    ob_line = np.arange(num_min, num_max, dmm / 30)
    plt.plot(ob_line, ob_line, '--')
    for line in range(new_Fo_shape[0]):
        if member_list is None:
            if new_Fo_shape[0] == 1:
                label = '预报'
            else:
                label = '预报' + str(line + 1)
        else:
            label = member_list[line]
        fo_sorted = np.sort(new_Fo[line, :].flatten())
        fo_sorted_smooth = fo_sorted
        fo_sorted_smooth[1:-1] = 0.5 * fo_sorted[1:-1] + 0.25 * (fo_sorted[0:-2] + fo_sorted[2:])
        fo_sorted_smooth = fo_sorted_smooth[start_index:end_index]
        plt.plot(fo_sorted_smooth, ob_sorted_smooth, linewidth=2, label=label)
        plt.xlim(num_min, num_max)
        plt.ylim(num_min, num_max)
        plt.xlabel("预报", fontsize=0.9 * sup_fontsize)
        plt.ylabel("观测", fontsize=0.9 * sup_fontsize)
        plt.title("频率匹配映射关系图", fontsize=0.9 * sup_fontsize)
        plt.legend(loc="lower right", fontsize=0.9 * sup_fontsize)
        plt.yticks(fontsize=0.8 * sup_fontsize)
        plt.xticks(fontsize=0.8 * sup_fontsize)
    if title is not None:
        plt.suptitle(title + "\n", y=1.00, fontsize=sup_fontsize)
    if grid:
        plt.grid()
    if save_path is None:
        show = True
    else:
        plt.savefig(save_path,bbox_inches='tight')
        print("检验结果已以图片形式保存至" + save_path)
    if show is True:
        plt.show()
    plt.close()


def box_plot_continue(ob, fo,  member_list=None,vmax = None,vmin = None, save_path=None, show = False,dpi = 300,title="频率对比箱须图",
                      sup_fontsize = 10,width = None,height = None,):
    '''
    box_plot 画一两组数据的箱型图
    ---------------
    :param Ob: 实况数据  任意维numpy数组
    :param Fo: 预测数据 任意维numpy数组,Fo.shape 和Ob.shape一致
    :param save_path: 图片保存路径，缺省时不输出图片，而是以默认绘图窗口形式展示
    :return:图片，包含箱须图，等级包括,横坐标为"观测"、"预报"，纵坐标为数据值
    '''
    Fo_shape = fo.shape
    Ob_shape = ob.shape
    Ob_shpe_list = list(Ob_shape)
    size = len(Ob_shpe_list)
    ind = -size
    Fo_Ob_index = list(Fo_shape[ind:])

    if Fo_Ob_index != Ob_shpe_list:
        print('预报数据和观测数据维度不匹配')
        return
    Ob_shpe_list.insert(0, -1)
    new_Fo_shape = tuple(Ob_shpe_list)
    new_Fo = fo.reshape(new_Fo_shape)
    new_Fo_shape = new_Fo.shape
    list_fo = list(new_Fo)

    xticks = ['观测']
    if member_list is None:
        if new_Fo_shape[0] == 1:
            xticks.append('预报')
        else:
            for i in range(new_Fo_shape[0]):
                xticks.append('预报' + str(i + 1))
    else:
        xticks.extend(member_list)

    #print(width)
    new_list_fo = []
    ob = ob.flatten()
    new_list_fo.append(ob)
    for fo_piece in list_fo:
        new_list_fo.append(fo_piece.flatten())

    tuple_of_ob = tuple(new_list_fo)
    if width is None:
        width = meteva.base.plot_tools.caculate_axis_width(xticks, sup_fontsize) +0.5
        if width >10:
            for i in range(len(xticks)):
                if i % 2 ==1:
                    xticks[i] ="|\n" + xticks[i]
            width = 10
        elif width < 5:
            width = 5

    if height is None:
        height = width/2

    fig = plt.figure(figsize=(width, height), dpi=dpi)
    markersize = 5 * width * height / np.sqrt(ob.size)
    if markersize < 1:
        markersize = 1
    elif markersize > 20:
        markersize = 20
    colors_list= []
    colors = cm.get_cmap('rainbow', 128)
    for i in range(len(xticks)):
        color_grade = i / len(xticks)
        colors_list.append(colors(color_grade))

    bplot = plt.boxplot(tuple_of_ob, showfliers=True, patch_artist=True, labels=xticks)


    plt.xticks(fontsize = 0.9 * sup_fontsize)
    plt.yticks(fontsize = 0.9 * sup_fontsize)

    plt.title(title,fontsize = sup_fontsize)
    for i, item in enumerate(bplot["boxes"]):
        item.set_facecolor(colors_list[i])
    #plt.title(title, fontsize=sup_fontsize)

    if vmin is not None or vmax is not None:
        if vmin is not None:
            if vmax is None:
                vmax = max(np.max(ob), np.max(fo))
                dmax = vmax - vmin
                plt.ylim(vmin,vmax+ dmax * 0.05)
            else:
                plt.ylim(vmin, vmax)
        else:
            vmin =  min(np.min(ob), np.min(fo))
            dmax = vmax - vmin
            plt.ylim(vmin- dmax * 0.05,vmax)


    if save_path is None:
        show = True
    else:
        plt.savefig(save_path,bbox_inches='tight')
        print("检验结果已以图片形式保存至" + save_path)
    if show:
        plt.show()
    plt.close()



'''
def box_plot_continue(ob, fo,  member_list=None,vmax = None,vmin = None, save_path=None, show = False,dpi = 300,title="频率对比箱须图",
                      sup_fontsize = 10,width = None,height = None,):
    
    #box_plot 画一两组数据的箱型图
    #---------------
    #:param Ob: 实况数据  任意维numpy数组
    #:param Fo: 预测数据 任意维numpy数组,Fo.shape 和Ob.shape一致
    #:param save_path: 图片保存路径，缺省时不输出图片，而是以默认绘图窗口形式展示
    #:return:图片，包含箱须图，等级包括,横坐标为"观测"、"预报"，纵坐标为数据值
    
    Fo_shape = fo.shape
    Ob_shape = ob.shape
    Ob_shpe_list = list(Ob_shape)
    size = len(Ob_shpe_list)
    ind = -size
    Fo_Ob_index = list(Fo_shape[ind:])

    if Fo_Ob_index != Ob_shpe_list:
        print('预报数据和观测数据维度不匹配')
        return
    Ob_shpe_list.insert(0, -1)
    new_Fo_shape = tuple(Ob_shpe_list)
    new_Fo = fo.reshape(new_Fo_shape)
    new_Fo_shape = new_Fo.shape
    list_fo = list(new_Fo)

    xticks = ['观测']
    if member_list is None:
        if new_Fo_shape[0] == 1:
            xticks.append('预报')
        else:
            for i in range(new_Fo_shape[0]):
                xticks.append('预报' + str(i + 1))
    else:
        xticks.extend(member_list)

    #print(width)
    new_list_fo = []
    ob = ob.flatten()
    new_list_fo.append(ob)
    for fo_piece in list_fo:
        new_list_fo.append(fo_piece.flatten())

    tuple_of_ob = tuple(new_list_fo)
    if width is None:
        width = meteva.base.plot_tools.caculate_axis_width(xticks, sup_fontsize) +0.5
        if width >10:
            for i in range(len(xticks)):
                if i % 2 ==1:
                    xticks[i] ="|\n" + xticks[i]
            width = 10
        elif width < 5:
            width = 5

    if height is None:
        height = width/2

    fig = plt.figure(figsize=(width, height), dpi=dpi)
    markersize = 5 * width * height / np.sqrt(ob.size)
    if markersize < 1:
        markersize = 1
    elif markersize > 20:
        markersize = 20
    colors_list= []
    colors = cm.get_cmap('rainbow', 128)
    for i in range(len(xticks)):
        color_grade = i / len(xticks)
        colors_list.append(colors(color_grade))

    bplot = plt.boxplot(tuple_of_ob, showfliers=True, patch_artist=True, labels=xticks)


    plt.xticks(fontsize = 0.9 * sup_fontsize)
    plt.yticks(fontsize = 0.9 * sup_fontsize)

    plt.title(title,fontsize = sup_fontsize)
    for i, item in enumerate(bplot["boxes"]):
        item.set_facecolor(colors_list[i])
    #plt.title(title, fontsize=sup_fontsize)

    if vmin is not None or vmax is not None:
        if vmin is not None:
            if vmax is None:
                vmax = max(np.max(ob), np.max(fo))
                dmax = vmax - vmin
                plt.ylim(vmin,vmax+ dmax * 0.05)
            else:
                plt.ylim(vmin, vmax)
        else:
            vmin =  min(np.min(ob), np.min(fo))
            dmax = vmax - vmin
            plt.ylim(vmin- dmax * 0.05,vmax)


    if save_path is None:
        show = True
    else:
        plt.savefig(save_path,bbox_inches='tight')
        print("检验结果已以图片形式保存至" + save_path)
    if show:
        plt.show()
    plt.close()

'''


def taylor_diagram(ob, fo,member_list=None, save_path=None,show = False,dpi = 300, title="",
                sup_fontsize =10,width = None,height = None):
    '''

    :param ob:
    :param fo:
    :param grade_list:
    :return:
    '''

    leftw = 0.3
    rightw = 1.5
    uphight = 1.2
    lowhight = 1.2
    axis_size_x = 3
    axis_size_y = 3
    if width is None:
        width = axis_size_x + leftw + rightw

    if height is None:
        height = axis_size_y + uphight + lowhight

    stds = meteva.method.ob_fo_std(ob,fo)
    corrs = meteva.method.corr(ob,fo)
    corrs1 = [1]
    if isinstance(corrs,float):
        corrs1.append(corrs)
    else:
        for i in range(len(corrs)):
            corrs1.append(corrs[i])


    fig = plt.figure(figsize=(width, height),dpi=dpi)
    ax1 = fig.add_axes([leftw / width, lowhight / width, axis_size_x / width, axis_size_y / height])


    max_stds = max(stds)

    dif = (max_stds) / 10.0
    if dif == 0:
        inte = 1
    else:
        inte = math.pow(10, math.floor(math.log10(dif)))
    # 用基本间隔，将最大最小值除于间隔后小数点部分去除，最后把间隔也整数化
    r = dif / inte

    if r < 2.3:
        inte = inte * 2
    elif r < 7:
        inte = inte * 5
    else:
        inte = inte * 10
    vmax = inte * ((int)(max_stds / inte) + 1)

    std_list = np.arange(inte,vmax+inte/2,inte)

    corr_list = [0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9,0.98]
    #画观测弧线
    # 画弧线
    angle = np.arange(0,math.pi/2,math.pi/1000)
    std = stds[0]
    x2 = np.cos(angle) * std
    y2 = np.sin(angle) * std
    plt.plot(x2, y2,color = "steelblue",linewidth = 1)
    for i in range(len(std_list)):
        std = std_list[i]
        x2 = np.cos(angle) * std
        y2 = np.sin(angle) * std
        if i <len(std_list)-1:
            plt.plot(x2, y2, ":", color="k", linewidth=0.5)
        else:
            plt.plot(x2, y2,color = "k",linewidth = 0.5)


    #画围绕观测的弧线
    angle = np.arange(0,math.pi,math.pi/1000)
    for i in range(len(std_list)):
        std = std_list[i]
        x2 = np.cos(angle) * std + stds[0]
        y2 = np.sin(angle) * std
        dis = np.sqrt(x2 * x2 + y2 * y2)
        x2 = x2[dis < vmax]
        y2 = y2[dis < vmax]

        plt.plot(x2, y2, ":", color="g", linewidth=0.5)

    #相关系数射线
    r0 = np.arange(0,vmax,inte/100)
    for i in range(len(corr_list)):
        corr = corr_list[i]
        angle = np.arccos(corr)
        x1 = r0 * np.cos(angle)
        y1 = r0 * np.sin(angle)
        ax1.plot(x1, y1, '-.', color='b', linewidth=0.4)
        rt = vmax* 1.01
        xt = rt * np.cos(angle)
        yt = rt * np.sin(angle)
        ax1.text(xt, yt, str(corr),fontsize = sup_fontsize * 0.8)


    angle = 60 * math.pi/180
    rt = vmax * 1.01
    xt = rt * np.cos(angle)
    yt = rt * np.sin(angle)

    ax1.text(xt, yt,"相关系数" , fontsize=sup_fontsize * 0.8,rotation=-30,)

    ax1.set_xticks(std_list)
    ax1.set_yticks(std_list)
    ax1.set_xlim(0,vmax)
    ax1.set_ylim(0, vmax)
    ax1.spines['right'].set_visible(False)
    ax1.spines['top'].set_visible(False)
    ax1.set_xlabel("标准差")
    ax1.set_ylabel("标准差")



    # 画预报观测点
    labels = ["观测"]
    labels.extend(member_list)
    for i in range(len(stds)):
        corr = corrs1[i]
        angle = np.arccos(corr)
        xp = stds[i] * np.cos(angle)
        yp = stds[i] * np.sin(angle)
        ax1.plot(xp, yp,"o", label=labels[i], markersize=6)
    lines, label1 = ax1.get_legend_handles_labels()

    if len(stds)> 7:
        legend2 = ax1.legend(lines, label1, loc="upper right",
                         bbox_to_anchor=(1.4, 1.05), ncol=1, fontsize=sup_fontsize * 0.9)
    elif len(stds)>5:
        legend2 = ax1.legend(lines, label1, loc="upper right",
                         bbox_to_anchor=(1.2, 1.05), ncol=1, fontsize=sup_fontsize * 0.9)
    else:
        legend2 = ax1.legend(lines, label1, loc="upper right",
                         bbox_to_anchor=(1.1, 1.05), ncol=1, fontsize=sup_fontsize * 0.9)
    ax1.add_artist(legend2)


    title = title + "\n"
    ax1.set_title(title,fontsize = sup_fontsize)
    if save_path is None:
        show = True
    else:
        plt.savefig(save_path,bbox_inches='tight')
        print("检验结果已以图片形式保存至" + save_path)
    if show is True:
        plt.show()
    plt.close()


def taylor_diagram_(ob, fo,member_list=None, save_path=None,show = False,dpi = 300, title="",
                sup_fontsize =10,width = None,height = None):
    '''

    :param ob:
    :param fo:
    :param grade_list:
    :return:
    '''

    leftw = 0.3
    rightw = 1.5
    uphight = 1.2
    lowhight = 1.2
    axis_size_x = 3
    axis_size_y = 3
    if width is None:
        width = axis_size_x + leftw + rightw

    if height is None:
        height = axis_size_y + uphight + lowhight

    stds = meteva.method.ob_fo_std(ob,fo)
    corrs = meteva.method.corr(ob,fo)
    corrs1 = [1]
    if isinstance(corrs,float):
        corrs1.append(corrs)
    else:
        for i in range(len(corrs)):
            corrs1.append(corrs[i])


    fig = plt.figure(figsize=(width, height),dpi=dpi)
    ax1 = fig.add_axes([leftw / width, lowhight / width, axis_size_x / width, axis_size_y / height])


    max_stds = max(stds)

    dif = (max_stds) / 10.0
    if dif == 0:
        inte = 1
    else:
        inte = math.pow(10, math.floor(math.log10(dif)))
    # 用基本间隔，将最大最小值除于间隔后小数点部分去除，最后把间隔也整数化
    r = dif / inte

    if r < 2.3:
        inte = inte * 2
    elif r < 7:
        inte = inte * 5
    else:
        inte = inte * 10
    vmax = inte * ((int)(max_stds / inte) + 1)

    std_list = np.arange(inte,vmax+inte/2,inte)

    corr_list = [0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9,0.98]
    #画观测弧线
    # 画弧线
    angle = np.arange(0,math.pi/2,math.pi/1000)
    std = stds[0]
    x2 = np.cos(angle) * std
    y2 = np.sin(angle) * std
    plt.plot(x2, y2,color = "steelblue",linewidth = 1)
    for i in range(len(std_list)):
        std = std_list[i]
        x2 = np.cos(angle) * std
        y2 = np.sin(angle) * std
        if i <len(std_list)-1:
            plt.plot(x2, y2, ":", color="k", linewidth=0.5)
        else:
            plt.plot(x2, y2,color = "k",linewidth = 0.5)


    #画围绕观测的弧线
    angle = np.arange(0,math.pi,math.pi/1000)
    for i in range(len(std_list)):
        std = std_list[i]
        x2 = np.cos(angle) * std + stds[0]
        y2 = np.sin(angle) * std
        dis = np.sqrt(x2 * x2 + y2 * y2)
        x2 = x2[dis < vmax]
        y2 = y2[dis < vmax]

        plt.plot(x2, y2, ":", color="g", linewidth=0.5)

    #相关系数射线
    r0 = np.arange(0,vmax,inte/100)
    for i in range(len(corr_list)):
        corr = corr_list[i]
        angle = np.arccos(corr)
        x1 = r0 * np.cos(angle)
        y1 = r0 * np.sin(angle)
        ax1.plot(x1, y1, '-.', color='b', linewidth=0.4)
        rt = vmax* 1.01
        xt = rt * np.cos(angle)
        yt = rt * np.sin(angle)
        ax1.text(xt, yt, str(corr),fontsize = sup_fontsize * 0.8)


    angle = 60 * math.pi/180
    rt = vmax * 1.01
    xt = rt * np.cos(angle)
    yt = rt * np.sin(angle)

    ax1.text(xt, yt,"相关系数" , fontsize=sup_fontsize * 0.8,rotation=-30,)

    ax1.set_xticks(std_list)
    ax1.set_yticks(std_list)
    ax1.set_xlim(0,vmax)
    ax1.set_ylim(0, vmax)
    ax1.spines['right'].set_visible(False)
    ax1.spines['top'].set_visible(False)
    ax1.set_xlabel("标准差")
    ax1.set_ylabel("标准差")



    # 画预报观测点
    labels = ["观测"]
    labels.extend(member_list)
    for i in range(len(stds)):
        corr = corrs1[i]
        angle = np.arccos(corr)
        xp = stds[i] * np.cos(angle)
        yp = stds[i] * np.sin(angle)
        ax1.plot(xp, yp,"o", label=labels[i], markersize=6)
    lines, label1 = ax1.get_legend_handles_labels()

    if len(stds)> 7:
        legend2 = ax1.legend(lines, label1, loc="upper right",
                         bbox_to_anchor=(1.4, 1.05), ncol=1, fontsize=sup_fontsize * 0.9)
    elif len(stds)>5:
        legend2 = ax1.legend(lines, label1, loc="upper right",
                         bbox_to_anchor=(1.2, 1.05), ncol=1, fontsize=sup_fontsize * 0.9)
    else:
        legend2 = ax1.legend(lines, label1, loc="upper right",
                         bbox_to_anchor=(1.1, 1.05), ncol=1, fontsize=sup_fontsize * 0.9)
    ax1.add_artist(legend2)


    title = title + "\n"
    ax1.set_title(title,fontsize = sup_fontsize)
    if save_path is None:
        show = True
    else:
        plt.savefig(save_path,bbox_inches='tight')
        print("检验结果已以图片形式保存至" + save_path)
    if show is True:
        plt.show()
    plt.close()



def frequency_histogram_error(ob, fo,grade_list=None, member_list=None,  vmax = None,save_path=None,show = False,dpi = 300,plot = "bar", title="误差频率统计图",
                        sup_fontsize = 10,width = None,height = None,log_y = False,color_list = None,linestyle = None):
    '''
    frequency_histogram 对比测试数据和实况数据的发生的频率
    :param ob: 实况数据 任意维numpy数组
    :param fo: 预测数据 任意维numpy数组,Fo.shape 和Ob.shape一致
    :param grade_list: 如果该参数为None，观测或预报值出现过的值都作为分类标记.
    如果该参数不为None，它必须是一个从小到大排列的实数，以其中列出的数值划分出的多个区间作为分类标签。
    对于预报和观测值不为整数的情况，grade_list 不能设置为None。
    :param save_path: 保存地址
    :return: 无
    '''
    Fo_shape = fo.shape
    Ob_shape = ob.shape
    Ob_shpe_list = list(Ob_shape)
    size = len(Ob_shpe_list)
    ind = -size
    Fo_Ob_index = list(Fo_shape[ind:])
    if Fo_Ob_index != Ob_shpe_list:
        print('预报数据和观测数据维度不匹配')
        return
    Ob_shpe_list.insert(0, -1)
    new_Fo_shape = tuple(Ob_shpe_list)
    new_Fo = fo.reshape(new_Fo_shape)
    new_Fo_shape = new_Fo.shape


    legend = []
    if member_list is None:
        if new_Fo_shape[0] <= 1:
            legend.append('预报')
        else:
            for i in range(new_Fo_shape[0]):
                legend.append('预报' + str(i + 1))
    else:
        legend.extend(member_list)

    error = new_Fo - ob

    result_array = meteva.method.frequency_table(ob, error, grade_list=grade_list)
    result_array = result_array[1:,:]

    total_count = np.sum(result_array[0,:])
    result_array /= total_count
    if grade_list is not None:
        if len(grade_list) >10:
            axis = ["<\n" + str(round(grade_list[0],6))]
            for index in range(len(grade_list)):
                axis.append(str(round(grade_list[index],6)))
            axis.append(">=\n" + str(round(grade_list[-1],6)))
        else:
            axis = ["<" + str(round(grade_list[0],6))]
            for index in range(len(grade_list) - 1):
                axis.append("[" + str(round(grade_list[index],6)) + "," + str(round(grade_list[index + 1],6)) + ")")
            axis.append(">=" + str(round(grade_list[-1],6)))


    else:
        new_fo = copy.deepcopy(fo).flatten()
        new_ob = copy.deepcopy(ob).flatten()
        fo_list = list(set(new_fo.tolist()))
        fo_list.extend(list(set(new_ob.tolist())))
        axis = list(set(fo_list))

    name_list_dict = {}
    name_list_dict["legend"] = legend
    name_list_dict["类别"] = axis
    if log_y:
        vmin = None
    else:
        vmin = 0
    if plot == "bar":
        meteva.base.plot_tools.bar(result_array,name_list_dict,ylabel= "样本占比",vmin = vmin,vmax = vmax,save_path = save_path,show = show,dpi = dpi,title=title,
                                   width = width,height = height,sup_fontsize= sup_fontsize,log_y = log_y,color_list=color_list)
    else:
        meteva.base.plot_tools.plot(result_array, name_list_dict, ylabel="样本占比", vmin=vmin, vmax=vmax, save_path=save_path,
                                   show=show, dpi=dpi, title=title,
                                    width = width,height = height,sup_fontsize= sup_fontsize,log_y = log_y,color_list=color_list,linestyle=linestyle)



def accumulation_change_with_strength(ob,fo,member_list = None,save_path=None,  show = False,dpi = 300,title="降水量随强度变化图",
             sup_fontsize = 14,width = None,height = None,log_y = False,y_log = None,max_x = None):
    if y_log is not None:
        print(
            "warning: the argument y_log will be abolished, please use log_y instead\n警告：为保持和其它函数的名称一致，参数log_y将被废除，以后请使用参数log_y代替")
        log_y = y_log

    accu_stren = meteva.method.continuous.table.accumulation_strength_table(ob,fo)
    min_not_zero = np.min(accu_stren[accu_stren>0])
    maxv = np.max(accu_stren)
    shape = accu_stren.shape
    grade = np.arange(1,shape[1]+1,1)
    nfo = shape[0]-1
    if width is None:
        width = 10
    if height is None:
        height = width *0.6

    fig = plt.figure(figsize=(width, height),dpi = dpi)

    if member_list is None:
        labels = ["观测"]
        for i in range(1,nfo+1):
            labels.append("预报"+str(i))
    else:
        labels = ["观测"]
        labels.extend(member_list)
    for line in range(len(labels)):
        plt.plot(grade,accu_stren[line], label=labels[line],marker = ".")
        plt.xlabel("降水强度(毫米/小时)", fontsize=0.9 * sup_fontsize)
        plt.ylabel("累计降水量", fontsize=0.9 * sup_fontsize)
        plt.title(title, fontsize=0.9 * sup_fontsize)
        if(log_y):
            ax_one = plt.gca()
            for tick in ax_one.yaxis.get_major_ticks():
                tick.label1.set_fontproperties('stixgeneral')
            plt.yscale('log')
        plt.xticks(fontsize=0.8 * sup_fontsize)
        plt.legend(loc="upper right")



    # 设置次刻度间隔
    if max_x is not None:
        maxx = max_x
    else:
        maxx = len(grade)
    if(maxx <20):
        xmi = 1
        Xmi = 1
    elif(maxx <50):
        xmi = 1
        Xmi = 5
    elif (maxx <100):
        xmi = 1
        Xmi = 10
    elif (maxx <300):
        xmi = 5
        Xmi = 20
    elif (maxx <1000):
        xmi = 10
        Xmi = 50
    else:
        xmi = 50
        Xmi = 200
    ax1 = plt.gca()
    xmajorLocator = mpl.ticker.MultipleLocator(Xmi)  # 将x主刻度标签设置为次刻度10倍
    ax1.xaxis.set_major_locator(xmajorLocator)
    xminorLocator = mpl.ticker.MultipleLocator(xmi)  # 将x轴次刻度标签设置xmi
    ax1.xaxis.set_minor_locator(xminorLocator)
    plt.xlim(0,maxx)
    if log_y:
        plt.ylim(min_not_zero,maxv * 3)
    else:
        plt.ylim(0,maxv * 1.1)
    if save_path is None:
        show = True
    else:
        plt.savefig(save_path, bbox_inches='tight')
        print("检验结果已以图片形式保存至" + save_path)
    if show is True:
        plt.show()
    plt.close()
    return accu_stren

def frequency_change_with_strength(ob,fo,member_list = None,save_path=None,  show = False,dpi = 300,title="降水频次随强度变化图",
             sup_fontsize = 14,width = None,height = None,log_y = False,y_log = None,max_x = None,color_list = None,linestyle = None,smooth = None):

    if y_log is not None:
        print(
            "warning: the argument y_log will be abolished, please use log_y instead\n警告：为保持和其它函数的名称一致，参数log_y将被废除，以后请使用参数log_y代替")
        log_y = y_log

    accu_stren = meteva.method.continuous.table.frequency_strength_table(ob,fo)
    if smooth is not None:
        for i in range(accu_stren.shape[0]):
            accu_stren1 = accu_stren[i,:]
            accu_stren_sm = accu_stren[i,:]
            for k in range(smooth):
                accu_stren_sm[10:-1] = 0.25 * accu_stren1[11:] + 0.25 * accu_stren1[9:-2] + 0.5 * accu_stren1[10:-1]
                accu_stren1[:] = accu_stren_sm[:]
            accu_stren[i, :]  = accu_stren_sm[:]

    min_not_zero = np.min(accu_stren[accu_stren>=1])
    maxv = np.max(accu_stren)
    shape = accu_stren.shape
    grade = np.arange(1,shape[1]+1,1)
    nfo = shape[0]-1
    if width is None:
        width = 10
    if height is None:
        height = width *0.6

    fig = plt.figure(figsize=(width, height),dpi = dpi)

    if member_list is None:
        labels = ["观测"]
        for i in range(1,nfo+1):
            labels.append("预报"+str(i))
    else:
        if len(member_list) == shape[0]:
            labels = member_list
        else:
            labels = ["观测"]
            labels.extend(member_list)

    for line in range(len(labels)):
        if linestyle is None:
            style = "-"
        else:
            style = linestyle[line]
        if color_list is None:
            plt.plot(grade,accu_stren[line], label=labels[line],marker = ".",linestyle=style)
        else:
            plt.plot(grade, accu_stren[line], label=labels[line], marker=".",color = color_list[line],linestyle = style)

    plt.xlabel("降水强度(毫米/小时)", fontsize=0.9 * sup_fontsize)
    plt.ylabel("降水频次", fontsize=0.9 * sup_fontsize)
    plt.title(title, fontsize=0.9 * sup_fontsize)
    if(log_y):
        ax_one = plt.gca()
        for tick in ax_one.yaxis.get_major_ticks():
            tick.label1.set_fontproperties('stixgeneral')
        plt.yscale('log')
    plt.xticks(fontsize=0.8 * sup_fontsize)
    plt.legend(loc="upper right")




    # 设置次刻度间隔
    if max_x is not None:
        maxx = max_x
    else:
        maxx = len(grade)
    if(maxx <20):
        xmi = 1
        Xmi = 1
    elif(maxx <50):
        xmi = 1
        Xmi = 5
    elif (maxx <100):
        xmi = 1
        Xmi = 10
    elif (maxx <300):
        xmi = 5
        Xmi = 20
    elif (maxx <1000):
        xmi = 10
        Xmi = 50
    else:
        xmi = 50
        Xmi = 200
    ax1 = plt.gca()
    xmajorLocator = mpl.ticker.MultipleLocator(Xmi)  # 将x主刻度标签设置为次刻度10倍
    ax1.xaxis.set_major_locator(xmajorLocator)
    xminorLocator = mpl.ticker.MultipleLocator(xmi)  # 将x轴次刻度标签设置xmi
    ax1.xaxis.set_minor_locator(xminorLocator)
    plt.xlim(0,maxx)
    if log_y:
        plt.ylim(min_not_zero,maxv * 3)
    else:
        plt.ylim(0,maxv * 1.1)
    if save_path is None:
        show = True
    else:
        plt.savefig(save_path, bbox_inches='tight')
        print("检验结果已以图片形式保存至" + save_path)
    if show is True:
        plt.show()
    plt.close()
    return accu_stren



def accumulation_change_with_strenght(ob,fo,member_list = None,save_path=None,  show = False,dpi = 300,title="降水量随强度变化图",
             sup_fontsize = 14,width = None,height = None,log_y = False,y_log = None,max_x = None):
    return accumulation_change_with_strength(ob,fo,member_list=member_list,save_path=save_path,show=show,dpi=dpi,title=title,
                                      sup_fontsize=sup_fontsize,width = width,height=height,log_y=log_y,y_log=y_log,max_x=max_x)

def frequency_change_with_strenght(ob,fo,member_list = None,save_path=None,  show = False,dpi = 300,title="降水频次随强度变化图",
             sup_fontsize = 14,width = None,height = None,log_y = False,y_log = None,max_x = None,color_list = None,linestyle = None,smooth = None):
    return  frequency_change_with_strength(ob,fo,member_list=member_list,save_path=save_path,show=show,dpi=dpi,title=title,sup_fontsize=sup_fontsize,width=width,height=height,
                                   log_y=log_y,y_log=y_log,max_x=max_x,color_list=color_list,linestyle=linestyle,smooth=smooth)


import copy


def hoods2dSetUpLists(input_object, which_methods, mat):
    out = {}
    if "mincvr" in which_methods:
        out['mincvr'] = {}
        out['mincvr']['pod'] = copy.copy(mat)
        out['mincvr']['far'] = copy.copy(mat)
        out['mincvr']['ets'] = copy.copy(mat)

    if "multi_event" in which_methods:
        out['multi_event'] = {}
        out['multi_event']['pod'] = copy.copy(mat)
        out['multi_event']['f'] = copy.copy(mat)
        out['multi_event']['hk'] = copy.copy(mat)

    if "fuzzy" in which_methods:
        out['fuzzy'] = {}
        out['fuzzy']['pod'] = copy.copy(mat)
        out['fuzzy']['far'] = copy.copy(mat)
        out['fuzzy']['ets'] = copy.copy(mat)

    if "joint" in which_methods:
        out['joint'] = {}
        out['joint']['pod'] = copy.copy(mat)
        out['joint']['far'] = copy.copy(mat)
        out['joint']['ets'] = copy.copy(mat)

    if "fss" in which_methods:
        out['fss'] = {}
        out['fss']['values'] = copy.copy(mat)
        out['fss']['fss_random'] = out['fss']['fss_uniform'] = 0

    if "pragmatic" in which_methods:
        out['pragmatic'] = {}
        out['pragmatic']['bs'] = copy.copy(mat)
        out['pragmatic']['bss'] = copy.copy(mat)

    return out



import numpy as np
import math


def zapsmall(x, digits = 7):
    x = np.array(x)
    ina = x is np.nan
    mx = max(abs(x[~ina]))
    round_digis = digits
    if mx > 0:
        round_digis = math.floor(max(0, digits - (np.log10(mx))))
    return np.round(x,  round_digis)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Nov 22 23:16:25 2022

@author: tangbuxing
"""
import numpy as np
import pandas as pd
#import sys
#sys.path.append(r'F:\Work\MODE')
from meteva.method.space.fuzzy_logic.lib import kernel2dsmooth
import meteva


def S1(grd_ob,grd_fo,nx = 10,ny = 10,sigma = 1, gradFUN = "KernelGradFUN"):
    out = {}
    x = grd_ob.values.squeeze()
    xhat = grd_fo.values.squeeze()
    Xgrad = kernel2dsmooth.kernel2dsmooth(x, kernel_type="LoG", nx=nx, ny=ny, sigma=sigma)
    Ygrad = kernel2dsmooth.kernel2dsmooth(xhat, kernel_type="LoG", nx=nx, ny=ny, sigma=sigma)

    denom = sum(np.sum(np.maximum(abs(Xgrad), abs(Ygrad)), axis=0))
    numer = sum(np.sum(abs(Ygrad - Xgrad), axis=0))

    grid1 = meteva.base.get_grid_of_data(grd_ob)
    ob_grad = meteva.base.grid_data(grid1,Xgrad)
    grid2 = meteva.base.get_grid_of_data(grd_fo)
    fo_grad = meteva.base.grid_data(grid2, Ygrad)
    out = {"s1":100 * numer / denom,"ob_grad":ob_grad,"fo_grad":fo_grad}


    return out

def S1_original(x, xhat, gradFUN = "KernelGradFUN"):
    out = {}
    
    Xgrad = kernel2dsmooth.kernel2dsmooth(x, kernel_type = "LoG", nx = 10, ny = 12, sigma = 1)
    Ygrad = kernel2dsmooth.kernel2dsmooth(xhat, kernel_type = "LoG", nx = 10, ny = 12, sigma = 1)
    
    denom = sum(np.sum(np.maximum(abs(Xgrad),abs(Ygrad)), axis = 0))
    numer = sum(np.sum(abs(Ygrad - Xgrad),axis = 0))
        
    out = 100 * numer/denom
    return out

if __name__ == '__main__':
    # x = pd.read_csv(r"H:\test_data\input\mem\s1\UKfcst6.csv")
    # xhat = pd.read_csv(r"H:\test_data\input\mem\s1\UKobs6.csv")
    # look=S1(x, xhat)
    import meteva.base as meb
    grid1 = meb.grid([100, 120, 0.05], [24, 40, 0.05])
    path_ob = r'H:\test_data\input\mem\mode\ob\rain03\20072611.000.nc'
    path_fo = r'H:\test_data\input\mem\mode\ec\rain03\20072608.003.nc'
    grd_ob = meb.read_griddata_from_nc(path_ob, grid=grid1, time="2020072611", dtime=0, data_name="OBS")
    grd_fo = meb.read_griddata_from_nc(path_fo, grid=grid1, time="2020072608", dtime=3, data_name="ECMWF")

    #obs_array = grd_ob.values.squeeze()
    #fst_array = grd_fo.values.squeeze()
    look=S1(grd_ob, grd_fo)
    print(look)

    meb.smooth()


import pandas as pd
import numpy as np
import meteva
from scipy.ndimage import convolve
from scipy.ndimage.filters import uniform_filter
import copy
#将两个站点数据信息进行合并，并去重。

def put_stadata_on_station(sta,station):
    sta_list = meteva.base.split(sta)
    sta1_list = []
    for sta0 in sta_list:
        #删除重复行
        sta1 = sta0.drop_duplicates(['id'])
        #先将数据合并
        df = pd.merge(station,sta1, on='id', how='left')
        #时间，时效和层次，采用sta的
        df.iloc[:, 0] = sta1.iloc[0, 0]
        df.iloc[:, 1] = sta1.iloc[0, 1]
        df.iloc[:, 2] = sta1.iloc[0, 2]

        #如果合并后sta对应的数据是缺省，则用station里的data0列补充
        columns = list(sta1.columns)
        len_c1 = len(columns)
        columns_m = list(df.columns)
        len_m = len(columns_m)
        for i in range(len_c1+5,len_m):
            name1 = columns_m[i]
            name2 = columns_m[6]
            df.loc[df[name1].isnull(), name1] = df[df[name1].isnull()][name2]


        #删除合并后多余的时空信息列
        len_s = len(list(station.columns))
        drop_col = list(df.columns[6:len_s+5])
        #print(drop_col)
        df.drop(drop_col, axis=1, inplace=True)
        #重新命名列名称
        df.columns = sta1.columns
        df.attrs = copy.deepcopy(sta.attrs)
        sta1_list.append(df)
    sta_new = pd.concat(sta1_list,axis=0)
    return sta_new


def smooth(grd,smooth_times = 1,used_coords = "xy"):
    if (grd is None):
        return None
    levels = grd["level"].values
    times = grd["time"].values
    dtimes = grd["dtime"].values
    members = grd["member"].values
    grid0 = meteva.base.basicdata.get_grid_of_data(grd)
    grd_new = meteva.base.grid_data(grid0)
    for i in range(len(levels)):
        for j in range(len(times)):
            for k in range(len(dtimes)):
                for m in range(len(members)):
                    dat = grd.values[m,i,j,k,:,:]
                    #print(dat.shape)
                    kernel = np.array([[0.0625, 0.125, 0.0625],
                                       [0.125, 0.25, 0.125],
                                       [0.0625, 0.125, 0.0625]])
                    for s in range(smooth_times):
                        dat = convolve(dat, kernel)

                    grd_new.values[m,i,j,k,:,:] = dat[:,:]
    return grd_new




def moving_avarage(grd, half_window_size):
    return moving_ave(grd,half_window_size)


def moving_ave(grd,half_window_size):
    # 该函数计算网格点附近矩形方框内的平均值
    # 使用同规格的场，确保网格范围和分辨率一致
    # window_size 窗口尺度

    levels = copy.deepcopy(grd["level"].values)
    times = copy.deepcopy(grd["time"].values)
    dtimes = copy.deepcopy(grd["dtime"].values)
    members = copy.deepcopy(grd["member"].values)
    grd1 = copy.deepcopy(grd)
    size = half_window_size * 2 + 1
    for i in range(len(levels)):
        for j in range(len(times)):
            for k in range(len(dtimes)):
                for m in range(len(members)):
                    dat = grd1.values[m, i, j, k, :, :]
                    # print(type(dat[0,0]))
                    dat1 = uniform_filter(dat, size=size)
                    grd1.values[m, i, j, k, :, :] = np.round(dat1[:, :], 10)
    return grd1


def moving_max(grd, half_window_size):
    # 该函数计算网格点附近矩形方框内的最大值
    # 使用同规格的场，确保网格范围和分辨率一致
    # window_size 窗口尺度
    levels = copy.deepcopy(grd["level"].values)
    times = copy.deepcopy(grd["time"].values)
    dtimes = copy.deepcopy(grd["dtime"].values)
    members = copy.deepcopy(grd["member"].values)
    grd1 = copy.deepcopy(grd)
    nlon = len(grd["lon"])
    nlat = len(grd["lat"])


    size = half_window_size * 2 + 1
    for i in range(len(levels)):
        for j in range(len(times)):
            for k in range(len(dtimes)):
                for m in range(len(members)):

                    dat = grd1.values[m,i,j,k,:,:]
                    vmin = np.min(dat)
                    dat_3d_lon = np.ones((size,nlat,nlon))*vmin
                    for p in range(-half_window_size,half_window_size+1):
                        ps0 = max(0,p)
                        pe0 = min(nlon,nlon + p)
                        dat_3d_lon[p,:,ps0 : pe0] = dat[:,(ps0 - p):(pe0-p)]
                    dat_max_lon = np.max(dat_3d_lon,axis=0)

                    dat_3d_lat = np.ones((size,nlat,nlon))*vmin
                    for p in range(-half_window_size,half_window_size+1):
                        ps0 = max(0,p)
                        pe0 = min(nlat,nlat + p)
                        dat_3d_lat[p,ps0 : pe0,:] = dat_max_lon[(ps0 - p):(pe0-p),:]
                    dat_max_lat = np.max(dat_3d_lat,axis=0)

                    grd1.values[m, i, j, k, :, :] = dat_max_lat[:, :]
                    # 首先在x方向做求最大
    return grd1



def moving_min(grd, half_window_size):
    # 该函数计算网格点附近矩形方框内的最小值
    # 使用同规格的场，确保网格范围和分辨率一致
    # window_size 窗口尺度
    levels = copy.deepcopy(grd["level"].values)
    times = copy.deepcopy(grd["time"].values)
    dtimes = copy.deepcopy(grd["dtime"].values)
    members = copy.deepcopy(grd["member"].values)
    grd1 = copy.deepcopy(grd)
    nlon = len(grd["lon"])
    nlat = len(grd["lat"])


    size = half_window_size * 2 + 1
    for i in range(len(levels)):
        for j in range(len(times)):
            for k in range(len(dtimes)):
                for m in range(len(members)):

                    dat = grd1.values[m,i,j,k,:,:]
                    vmax = np.max(dat)
                    dat_3d_lon = np.ones((size,nlat,nlon))*vmax
                    for p in range(-half_window_size,half_window_size+1):
                        ps0 = max(0,p)
                        pe0 = min(nlon,nlon + p)
                        dat_3d_lon[p,:,ps0 : pe0] = dat[:,(ps0 - p):(pe0-p)]
                    dat_max_lon = np.min(dat_3d_lon,axis=0)

                    dat_3d_lat = np.ones((size,nlat,nlon))*vmax
                    for p in range(-half_window_size,half_window_size+1):
                        ps0 = max(0,p)
                        pe0 = min(nlat,nlat + p)
                        dat_3d_lat[p,ps0 : pe0,:] = dat_max_lon[(ps0 - p):(pe0-p),:]
                    dat_max_lat = np.min(dat_3d_lat,axis=0)

                    grd1.values[m, i, j, k, :, :] = dat_max_lat[:, :]
                    # 首先在x方向做求最大
    return grd1


def moving_std(grd,half_window_size):
    # 该函数计算网格点附近矩形方框内的标准差
    # 使用同规格的场，确保网格范围和分辨率一致
    # window_size 窗口尺度
    grd_move_mean = moving_avarage(grd,half_window_size)
    delta = copy.deepcopy(grd)
    delta.values[:] = np.power(grd.values[:] - grd_move_mean.values[:],2)
    delta =moving_avarage(delta,half_window_size)
    delta.values = np.sqrt(delta.values)

    return delta



#将两个站点dataframe相加在一起
def add_on_level_time_dtime_id(sta1,sta2,how = "left",default = None):
    if sta1 is None or len(sta1.index)==0:
        return sta2
    elif sta2 is None or len(sta2.index)==0:
        return sta1
    else:
        # 删除重复行
        dtime_type = str(sta1["dtime"].dtype)
        if dtime_type.find("int") < 0:
            print("警告：站点数据的dtime坐标不是整数，可能导致数据后续数据匹配失败，请检查输入数据，将站点数据的dtime坐标设置为整数类型")
            # return None
        df = pd.merge(sta1, sta2, on=["level","time","dtime","id"], how=how)
        # print(len(df.index))
        # df.drop_duplicates(subset=["level","time","dtime","id"],inplace=True)
        # print(len(df.index))

        #print(len(sta1.index))
        #print(len(sta2.index))
        #print(len(df.index))
        #时间，时效和层次，采用df1的

        #站点取df1，df2中非缺省的
        columns = list(sta1.columns)
        len_c1 = len(columns)
        columns_m = list(df.columns)
        #print(columns_m)
        for i in range(4,6):
            name1 = columns_m[i]
            name2 = columns_m[i+len_c1-4]
            df.loc[df[name1].isnull(), name1] = df[df[name1].isnull()][name2]


        #删除合并后第二组时空坐标信
        drop_col = list(df.columns[len_c1:len_c1+2])
        df.drop(drop_col, axis=1, inplace=True)

        #print(df)
        #相加前是否要先设定缺省值
        columns_m = list(df.columns)
        len_m = len(columns_m)
        if default is not None:
            for i in range(6, len_m):
                df.iloc[:, i].fillna(default, inplace=True)

        #对数据列进行相加
        len_d = len_m - len_c1

        for i in range(6,len_c1):
            df[df.columns.values[i]] = df.iloc[:, i] + df.iloc[:, i+len_d]

        #print(df)
        #删除df2对应的数据列
        columns_drop = list(df.columns[len_c1:len_m])
        df.drop(columns_drop, axis=1, inplace=True)

        #重新命名列名称
        df.columns = columns
        df.attrs = copy.deepcopy(sta1.attrs)
        return df


#将两个站点dataframe相乘

def divide_on_level_time_dtime_id(sta1,sta2,how = "left",default = None):

    sta2_ = copy.deepcopy(sta2)
    sta2_.iloc[:,6:] = 1/ sta2.iloc[:,6:]
    sta_div = mutiply_on_level_time_dtime_id(sta1,sta2_)
    return sta_div


def mutiply_on_level_time_dtime_id(sta1,sta2,how = "left",default = None):
    if sta1 is None:
        return sta2
    elif sta2 is None:
        return sta1
    else:
        # 删除重复行
        dtime_type = str(sta1["dtime"].dtype)
        if dtime_type.find("int") < 0:
            print("警告：站点数据的dtime坐标不是整数，可能导致数据后续数据匹配失败，请检查输入数据，将站点数据的dtime坐标设置为整数类型")
            # return None
        df = pd.merge(sta1, sta2, on=["level","time","dtime","id"], how=how)
        #print(len(sta1.index))
        #print(len(sta2.index))
        #print(len(df.index))
        #时间，时效和层次，采用df1的

        #站点取df1，df2中非缺省的
        columns = list(sta1.columns)
        len_c1 = len(columns)
        columns_m = list(df.columns)
        #print(columns_m)
        for i in range(4,6):
            name1 = columns_m[i]
            name2 = columns_m[i+len_c1-4]
            df.loc[df[name1].isnull(), name1] = df[df[name1].isnull()][name2]


        #删除合并后第二组时空坐标信
        drop_col = list(df.columns[len_c1:len_c1+2])
        df.drop(drop_col, axis=1, inplace=True)

        #print(df)
        #相加前是否要先设定缺省值
        columns_m = list(df.columns)
        len_m = len(columns_m)
        if default is not None:
            for i in range(6, len_m):
                df.iloc[:, i].fillna(default, inplace=True)

        #对数据列进行相乘
        len_d = len_m - len_c1
        for i in range(6,len_c1):
            df[df.columns.values[i]] = df.iloc[:, i] * df.iloc[:, i+len_d]

        #print(df)
        #删除df2对应的数据列
        columns_drop = list(df.columns[len_c1:len_m])
        df.drop(columns_drop, axis=1, inplace=True)

        #重新命名列名称
        df.columns = columns
        df.attrs = copy.deepcopy(sta1.attrs)
        return df


#将两个站点dataframe相减
def minus_on_level_time_dtime_id(sta1,sta2,how = "left",default = None):
    if sta1 is None:
        return sta2
    elif sta2 is None:
        return sta1
    else:
        # 删除重复行
        dtime_type = str(sta1["dtime"].dtype)
        if dtime_type.find("int") < 0:
            print("警告：站点数据的dtime坐标不是整数，可能导致数据后续数据匹配失败，请检查输入数据，将站点数据的dtime坐标设置为整数类型")
            # return None
        df = pd.merge(sta1, sta2, on=["level","time","dtime","id"], how=how)
        #print(len(sta1.index))
        #print(len(sta2.index))
        #print(len(df.index))
        #时间，时效和层次，采用df1的

        #站点取df1，df2中非缺省的
        columns = list(sta1.columns)
        len_c1 = len(columns)
        columns_m = list(df.columns)
        #print(columns_m)
        for i in range(4,6):
            name1 = columns_m[i]
            name2 = columns_m[i+len_c1-4]
            df.loc[df[name1].isnull(), name1] = df[df[name1].isnull()][name2]


        #删除合并后第二组时空坐标信
        drop_col = list(df.columns[len_c1:len_c1+2])
        df.drop(drop_col, axis=1, inplace=True)

        #print(df)
        #相加前是否要先设定缺省值
        columns_m = list(df.columns)
        len_m = len(columns_m)
        if default is not None:
            for i in range(6, len_m):
                df.iloc[:, i].fillna(default, inplace=True)

        #对数据列进行相减
        len_d = len_m - len_c1
        for i in range(6,len_c1):
            df[df.columns.values[i]] = df.iloc[:, i] - df.iloc[:, i+len_d]

        #print(df)
        #删除df2对应的数据列
        columns_drop = list(df.columns[len_c1:len_m])
        df.drop(columns_drop, axis=1, inplace=True)

        #重新命名列名称
        df.columns = columns
        df.attrs = copy.deepcopy(sta1.attrs)
        return df

def max_on_level_time_dtime_id(sta1,sta2,how = "left",default = None):
    if sta1 is None:
        return sta2
    elif sta2 is None:
        return sta1
    else:
        # 删除重复行
        dtime_type = str(sta1["dtime"].dtype)
        if dtime_type.find("int") < 0:
            print("警告：站点数据的dtime坐标不是整数，可能导致数据后续数据匹配失败，请检查输入数据，将站点数据的dtime坐标设置为整数类型")
            # return None
        df = pd.merge(sta1, sta2, on=["level","time","dtime","id"], how=how)
        #print(len(sta1.index))
        #print(len(sta2.index))
        #print(len(df.index))
        #时间，时效和层次，采用df1的

        #站点取df1，df2中非缺省的
        columns = list(sta1.columns)
        len_c1 = len(columns)
        columns_m = list(df.columns)
        #print(columns_m)
        for i in range(4,6):
            name1 = columns_m[i]
            name2 = columns_m[i+len_c1-4]
            df.loc[df[name1].isnull(), name1] = df[df[name1].isnull()][name2]


        #删除合并后第二组时空坐标信
        drop_col = list(df.columns[len_c1:len_c1+2])
        df.drop(drop_col, axis=1, inplace=True)

        #print(df)
        #相加前是否要先设定缺省值
        columns_m = list(df.columns)
        len_m = len(columns_m)
        if default is not None:
            for i in range(6, len_m):
                df.iloc[:, i].fillna(default, inplace=True)

        #对数据列进行相加
        len_d = len_m - len_c1
        for i in range(6,len_c1):

            values = df[[df.columns[i],df.columns[i+len_d]]].values
            max_value = values.max(axis = 1)
            df[df.columns.values[i]] = max_value

        #print(df)
        #删除df2对应的数据列
        columns_drop = list(df.columns[len_c1:len_m])
        df.drop(columns_drop, axis=1, inplace=True)

        #重新命名列名称
        df.columns = columns
        df.attrs = copy.deepcopy(sta1.attrs)
        return df


def min_on_level_time_dtime_id(sta1,sta2,how = "left",default = None):
    if sta1 is None:
        return sta2
    elif sta2 is None:
        return sta1
    else:
        # 删除重复行
        dtime_type = str(sta1["dtime"].dtype)
        if dtime_type.find("int") < 0:
            print("警告：站点数据的dtime坐标不是整数，可能导致数据后续数据匹配失败，请检查输入数据，将站点数据的dtime坐标设置为整数类型")
            # return None
        df = pd.merge(sta1, sta2, on=["level","time","dtime","id"], how=how)
        #print(len(sta1.index))
        #print(len(sta2.index))
        #print(len(df.index))
        #时间，时效和层次，采用df1的

        #站点取df1，df2中非缺省的
        columns = list(sta1.columns)
        len_c1 = len(columns)
        columns_m = list(df.columns)
        #print(columns_m)
        for i in range(4,6):
            name1 = columns_m[i]
            name2 = columns_m[i+len_c1-4]
            df.loc[df[name1].isnull(), name1] = df[df[name1].isnull()][name2]


        #删除合并后第二组时空坐标信
        drop_col = list(df.columns[len_c1:len_c1+2])
        df.drop(drop_col, axis=1, inplace=True)

        #print(df)
        #相加前是否要先设定缺省值
        columns_m = list(df.columns)
        len_m = len(columns_m)
        if default is not None:
            for i in range(6, len_m):
                df.iloc[:, i].fillna(default, inplace=True)

        #对数据列进行相加
        len_d = len_m - len_c1
        for i in range(6,len_c1):

            values = df[[df.columns[i],df.columns[i+len_d]]].values
            min_value = values.min(axis = 1)
            df[df.columns.values[i]] = min_value

        #print(df)
        #删除df2对应的数据列
        columns_drop = list(df.columns[len_c1:len_m])
        df.drop(columns_drop, axis=1, inplace=True)

        #重新命名列名称
        df.columns = columns
        df.attrs = copy.deepcopy(sta1.attrs)
        return df


def add_on_id(sta1_0, sta2_0, how="left", default=None):
    if sta1_0 is None:
        return sta2_0
    elif sta2_0 is None:
        return sta1_0
    else:
        # 删除重复行
        dtime_type = str(sta1_0["dtime"].dtype)
        if dtime_type.find("int") < 0:
            print("警告：站点数据的dtime坐标不是整数，可能导致数据后续数据匹配失败，请检查输入数据，将站点数据的dtime坐标设置为整数类型")
            # return None
        sta2 = sta2_0.drop_duplicates(['id'])
        sta1 = sta1_0.drop_duplicates(['id'])

        df = pd.merge(sta1, sta2, on='id', how=how)
        #print(len(sta1.index))
        #print(len(sta2.index))
        #print(len(df.index))
        #时间，时效和层次，采用df1的
        df.iloc[:, 0] = df.iloc[0, 0]
        df.iloc[:, 1] = df.iloc[0, 1]
        df.iloc[:, 2] = df.iloc[0, 2]

        #站点取df1，df2中非缺省的
        columns = list(sta1.columns)
        len_c1 = len(columns)
        columns_m = list(df.columns)
        for i in range(4,6):
            name1 = columns_m[i]
            name2 = columns_m[i+len_c1-1]
            df.loc[df[name1].isnull(), name1] = df[df[name1].isnull()][name2]


        #删除合并后第二组时空坐标信
        drop_col = list(df.columns[len_c1:len_c1+5])
        df.drop(drop_col, axis=1, inplace=True)

        #print(df)
        #相加前是否要先设定缺省值
        columns_m = list(df.columns)
        len_m = len(columns_m)
        if default is not None:
            for i in range(6, len_m):
                df.iloc[:, i].fillna(default, inplace=True)

        #对数据列进行相加
        len_d = len_m - len_c1
        for i in range(6,len_c1):
            df[df.columns.values[i]] = df.iloc[:, i] + df.iloc[:, i+len_d]

        #print(df)
        #删除df2对应的数据列
        columns_drop = list(df.columns[len_c1:len_m])
        df.drop(columns_drop, axis=1, inplace=True)

        #重新命名列名称
        df.columns = columns
        df.attrs = copy.deepcopy(sta1_0.attrs)
        return df

def max_on_id(sta1_0, sta2_0, how="left"):
    if sta1_0 is None:
        return sta2_0
    elif sta2_0 is None:
        return sta1_0
    else:
        # 删除重复行
        dtime_type = str(sta1_0["dtime"].dtype)
        if dtime_type.find("int") < 0:
            print("警告：站点数据的dtime坐标不是整数，可能导致数据后续数据匹配失败，请检查输入数据，将站点数据的dtime坐标设置为整数类型")
            # return None
        sta2 = sta2_0.drop_duplicates(['id'])
        sta1 = sta1_0.drop_duplicates(['id'])


        df = pd.merge(sta1, sta2, on='id', how=how)
        df.iloc[:, 0] = df.iloc[0, 0]
        df.iloc[:, 1] = df.iloc[0, 1]
        df.iloc[:, 2] = df.iloc[0, 2]
        #站点取df1，df2中非缺省的
        columns = list(sta1.columns)
        len_c1 = len(columns)
        columns_m = list(df.columns)
        for i in range(4,6):
            name1 = columns_m[i]
            name2 = columns_m[i+len_c1]
            df.loc[df[name1].isnull(), name1] = df[df[name1].isnull()][name2]

        #删除合并后第二组时空坐标信息
        drop_col = list(df.columns[len_c1:len_c1+5])
        df.drop(drop_col, axis=1, inplace=True)

        #对数据列判断最大
        datas = df.iloc[:,6:].values
        datas[datas == meteva.base.IV] = -meteva.base.IV
        maxdata = np.max(datas,axis=1)
        df.iloc[:,6] = maxdata
        #print(df)
        #删除df2对应的数据列
        len_m = len(df.columns)
        columns_drop = list(df.columns[7:len_m])
        df.drop(columns_drop, axis=1, inplace=True)
        #重新命名列名称
        df.columns = columns
        df.attrs = copy.deepcopy(sta1_0.attrs)
        return df

def min_on_id(sta1_0, sta2_0, how="left"):
    if sta1_0 is None:
        return sta2_0
    elif sta2_0 is None:
        return sta1_0
    else:
        # 删除重复行
        dtime_type = str(sta1_0["dtime"].dtype)
        if dtime_type.find("int") < 0:
            print("警告：站点数据的dtime坐标不是整数，可能导致数据后续数据匹配失败，请检查输入数据，将站点数据的dtime坐标设置为整数类型")
            # return None
        sta2 = sta2_0.drop_duplicates(['id'])
        sta1 = sta1_0.drop_duplicates(['id'])
        df = pd.merge(sta1, sta2, on='id', how=how)
        df.iloc[:, 0] = df.iloc[0, 0]
        df.iloc[:, 1] = df.iloc[0, 1]
        df.iloc[:, 2] = df.iloc[0, 2]
        #站点取df1，df2中非缺省的
        columns = list(sta1.columns)
        len_c1 = len(columns)
        columns_m = list(df.columns)
        for i in range(4,6):
            name1 = columns_m[i]
            name2 = columns_m[i+len_c1]
            df.loc[df[name1].isnull(), name1] = df[df[name1].isnull()][name2]

        #删除合并后第二组时空坐标信息
        drop_col = list(df.columns[len_c1:len_c1+5])
        df.drop(drop_col, axis=1, inplace=True)

        #对数据列判断最大
        datas = df.iloc[:,6:].values
        mindata = np.min(datas,axis=1)
        df.iloc[:,6] = mindata
        #print(df)
        #删除df2对应的数据列
        len_m = len(df.columns)
        columns_drop = list(df.columns[7:len_m])
        df.drop(columns_drop, axis=1, inplace=True)
        #重新命名列名称
        df.columns = columns
        df.attrs = copy.deepcopy(sta1_0.attrs)
        return df


#两个站点dataframe相减
def minus_on_id(sta1_0, sta2_0, how="left", default=None):

    # 删除重复行
    dtime_type = str(sta1_0["dtime"].dtype)
    if dtime_type.find("int") < 0:
        print("警告：站点数据的dtime坐标不是整数，可能导致数据后续数据匹配失败，请检查输入数据，将站点数据的dtime坐标设置为整数类型")
        #return None
    sta2 = sta2_0.drop_duplicates(['id'])
    sta1 = sta1_0.drop_duplicates(['id'])
    #将两个df1 合并在一起
    df = pd.merge(sta1, sta2, on='id', how=how)

    #时间，时效和层次，采用df1的
    df.iloc[:, 0] = df.iloc[0, 0]
    df.iloc[:, 1] = df.iloc[0, 1]
    df.iloc[:, 2] = df.iloc[0, 2]

    #站点取df1，df2中非缺省的
    columns = list(sta1.columns)
    len_c1 = len(columns)
    columns_m = list(df.columns)
    for i in range(4,6):
        name1 = columns_m[i]
        name2 = columns_m[i+len_c1]
        df.loc[df[name1].isnull(), name1] = df[df[name1].isnull()][name2]

    #删除合并后第二组时空坐标信息
    drop_col = list(df.columns[len_c1:len_c1+5])
    df.drop(drop_col, axis=1, inplace=True)

    #相加前是否要先设定缺省值
    columns_m = list(df.columns)
    len_m = len(columns_m)
    if default is not None:
        for i in range(6, len_m):
            df.iloc[:, i].fillna(default, inplace=True)

    #对数据列进行相加
    len_d = len_m - len_c1
    for i in range(6,len_c1):
        df[df.columns.values[i]] = df.iloc[:, i] - df.iloc[:, i+len_d]

    #删除df2对应的数据列
    columns_drop = list(df.columns[len_c1:len_m])
    df.drop(columns_drop, axis=1, inplace=True)

    #重新命名列名称
    df.columns = columns
    df.attrs = copy.deepcopy(sta1_0.attrs)
    return df


#两个dataframe相乘
def multiply_on_id(sta1_0, sta2_0, how="left", default=None):

    dtime_type = str(sta1_0["dtime"].dtype)
    if dtime_type.find("int") < 0:
        print("警告：站点数据的dtime坐标不是整数，可能导致数据后续数据匹配失败，请检查输入数据，将站点数据的dtime坐标设置为整数类型")
        #return None
    # 删除重复行
    sta2 = sta2_0.drop_duplicates(['id'])
    sta1 = sta1_0.drop_duplicates(['id'])

    #将两个df1 合并在一起
    df = pd.merge(sta1, sta2, on='id', how=how)

    #时间，时效和层次，采用df1的
    df.iloc[:, 0] = df.iloc[0, 0]
    df.iloc[:, 1] = df.iloc[0, 1]
    df.iloc[:, 2] = df.iloc[0, 2]

    #站点取df1，df2中非缺省的
    columns = list(sta1.columns)
    len_c1 = len(columns)
    columns_m = list(df.columns)
    for i in range(4,6):
        name1 = columns_m[i]
        name2 = columns_m[i+len_c1]
        df.loc[df[name1].isnull(), name1] = df[df[name1].isnull()][name2]

    #删除合并后第二组时空坐标信息
    drop_col = list(df.columns[len_c1:len_c1+5])
    df.drop(drop_col, axis=1, inplace=True)

    #相加前是否要先设定缺省值
    columns_m = list(df.columns)
    len_m = len(columns_m)
    if default is not None:
        for i in range(6, len_m):
            df.iloc[:, i].fillna(default, inplace=True)

    #对数据列进行相加
    len_d = len_m - len_c1
    for i in range(6,len_c1):
        df[df.columns.values[i]] = df.iloc[:, i] * df.iloc[:, i+len_d]

    #删除df2对应的数据列
    columns_drop = list(df.columns[len_c1:len_m])
    df.drop(columns_drop, axis=1, inplace=True)

    #重新命名列名称
    df.columns = columns
    df.attrs = copy.deepcopy(sta1_0.attrs)
    return df

def reset_value_as_IV(sta,iv_value):
    sta1 = sta.copy()
    data_names = meteva.base.get_stadata_names(sta1)
    for name in data_names:
        sta1.loc[sta1.loc[:,name] == iv_value,name] = meteva.base.IV
    return sta1


def get_ob_from_combined_data(sta_all,ob_column = 1):
    data_names = meteva.base.get_stadata_names(sta_all)
    sta_ob = meteva.base.sele_by_para(sta_all,member=data_names[:ob_column])
    dtime = sta_ob["dtime"].values[:]
    sta_ob["time"] = sta_ob["time"] + dtime * np.timedelta64(1, 'h')
    sta_ob["dtime"] = 0
    sta_ob = sta_ob.drop_duplicates(keep="first")
    return sta_ob




from meteva.base.fun.grouping import *
from meteva.product.program.fun import *
from meteva.base.fun.selecting import *
import numpy as np

def table(sta_ob_and_fos0,method,s = None,g = None,gll = None,save_dir = None,**kwargs):
    if s is not None:
        if g is not None:
            if g == "last_range" or g == "last_step":
                s["drop_last"] = False
            else:
                s["drop_last"] = True
    sta_ob_and_fos = sele_by_dict(sta_ob_and_fos0, s)
    if(len(sta_ob_and_fos.index) == 0):
        print("there is no data to verify")
        return
    sta_ob_and_fos_list,gll1 = group(sta_ob_and_fos,g,gll)
    data_name = meteva.base.get_stadata_names(sta_ob_and_fos_list[0])
    ensemble_score_method = [meteva.method.cr]
    group_num = len(sta_ob_and_fos_list)

    valid_gll = []
    result_all = []
    if type(method) == str:
        method =  globals().get(method)
    if method in ensemble_score_method:
        pass
    else:
        for i in range(group_num):
            sta = sta_ob_and_fos_list[i]
            if(len(sta.index) == 0):
                pass
            else:
                valid_group_list = None
                if gll1 is None:
                    valid_gll = None
                else:
                    valid_group_list = gll1[i]
                    valid_gll.append(gll1[i])
                ob = sta[data_name[0]].values
                fo = sta[data_name[1:]].values.T

                kwargs1 = copy.deepcopy(kwargs)
                if "save_path" not in kwargs.keys():
                    kwargs1["save_path"] = get_save_path(save_dir,method,g,valid_group_list,type = ".xlsx")
                kwargs1["member_list"] = data_name[1:]
                result1 = method(ob, fo, **kwargs1)
                result_all.append(result1)
        result_all = np.array(result_all)
    result = result_all.squeeze()

    return result,valid_gll





import meteva 
import datetime
import pandas as pd
import os 
import numpy as np


para = {
    "mid_method":meteva.method.tase,
    "grade_list": None,
    "compare":None,
    "middle_result_path": r"H:\test_data\output\mps\tase_z.h5",
    "begin_time": datetime.datetime(2018,8,1,0),
    "end_time": datetime.datetime(2018,8,2,0),
    "time_type":"UT",
    "time_step":12,     #起报时间间隔
    "grid" :  meteva.base.grid([70,140,0.25],[10,60,0.25]),  # 检验区域
    "level_list":  [ 850, 700, 500],
    "step":5,  #masker间隔（单位：°）
    "recover":False,
    "ob_data": {
        "CRA40":{
        "dirm": r"\\10.28.16.234\data2\AI\CRA40\2018\Z\LLL\YYYYMMDDHH.nc",  # 实况场数据路径
        "read_method": meteva.base.read_griddata_from_nc,
        "read_para": {},
        "time_type": "UT",  # 实况数据时间类型
        "multiple": 1,  # 当文件数据的单位是位势米，通过乘以9.8转换成位势
        }
    },
    "fo_data": {
        "Fu":{
            "dirm": r"\\10.28.16.234\data2\AI\fuxi\Z\LLL\YYYYMMDDHH\YYYYMMDDHH.TTT.nc",
            "dtime": [12, 240, 12],
            "read_method": meteva.base.read_griddata_from_nc,
            "read_para": {},
            "time_type": "UT",  # 预报数据时间类型是北京时，即08时起报
            "multiple": 1,  # 当文件数据的单位是位势米，通过乘以9.8转换成位势
            "move_fo_time":0,
            "veri_by":"CRA40"
        },

        "pa": {
            "dirm":   r"\\10.28.16.234\data2\AI\Pangu_ERA5\Z\LLL\YYYYMMDDHH\YYYYMMDDHH.TTT.nc",
            "dtime": [12, 240, 12],
            "read_method": meteva.base.read_griddata_from_nc,
            "read_para": {},
            "time_type": "UT",#预报数据时间类型是北京时，即08时起报
            "multiple":1, #当文件数据的单位是位势米，通过乘以9.8转换成位势
            "move_fo_time": 12,
            "veri_by": "CRA40"
        },
    },
}


def middle_of_score(para):
    middle_result_path = para["middle_result_path"]
    meteva.base.creat_path(middle_result_path)
    model_name_list = list(para["fo_data"].keys())
    mid_method = para["mid_method"]
    grid0 = para["grid"]
    # grid0 = meteva.base.grid([0,359.75,0.25],[-89.75,89.75,0.25])
    marker = meteva.perspact.get_grid_marker(grid0, step=para["step"])
    # marker = None
    level_list =para["level_list"]
    mid_list = []
    # 创建一个空的DataFrame，用来记录哪些中间量已经收集
    df1 = pd.DataFrame(data=None, columns=['level', 'time', 'dtime', 'member'])
    if os.path.exists(middle_result_path):
        if not para["recover"]:
            mid0 = pd.read_hdf(middle_result_path)
            mid_list.append(mid0)
            df1 = mid0[["level", "time", "dtime", "member"]]  # 如果有文件就替换掉df1
            df1.drop_duplicates(keep="first", inplace=True)  # 用来记录哪些时次已经收集了中间量

    begin_time = para["begin_time"]
    end_time = para["end_time"]
    if para["time_type"].lower()=="bt":
        begin_time -= datetime.timedelta(hours=8)
        end_time -= datetime.timedelta(hours=8)


    for model in model_name_list:
        df1_= df1.loc[df1['member'].isin([model])]
        time1 = begin_time
        para_fo1 = para["fo_data"][model]
        move_fo_time = para_fo1["move_fo_time"]
        veri_by = para_fo1["veri_by"]
        if veri_by =="self":
            para_ob1 = para_fo1
        else:
            para_ob1 = para["ob_data"][veri_by]

        save_k = 0  # 用来在收集的过程间隔一段时间报错一下结果，避免程序错误时没有任何输出导致要重头再来
        dtime_list = np.arange(para_fo1["dtime"][0], para_fo1["dtime"][1] + 1, para_fo1["dtime"][2]).tolist()
        while time1 <= end_time:
            df2 = meteva.base.in_time_list(df1_, time1)

            if len(df2.index) ==len(dtime_list):
                print(meteva.base.get_path(model + "YYYY年MM月DD日HH时起报的检验中间量已存在",time1))
            else:
                for dh in dtime_list:
                    df3 = meteva.base.in_dtime_list(df2, dtime_list=dh)
                    for level in level_list:
                        df4 = meteva.base.in_level_list(df3, level_list=level)
                        if len(df4.index) ==0:
                            save_k += 1  #
                            time_ob = time1 + datetime.timedelta(hours=dh)
                            time_ob_bt = time_ob + datetime.timedelta(hours=8)
                            if para_ob1["time_type"].lower() == "bt":
                                time_path = time_ob_bt
                            else:
                                time_path = time_ob

                            path0 = meteva.base.get_path(para_ob1["dirm"], time_path)
                            path0 = path0.replace("LLL",str(level))
                            if not os.path.exists(path0):
                                print(path0 +" not exist")
                                continue
                            grd_obs = para_ob1["read_method"](path0, grid=para["grid"], time=time_ob,
                                                                         dtime=0, data_name="OBS", show=True,level = level,
                                                                         **para_ob1["read_para"])
                            if grd_obs is None:
                                print("faild to read "+ path0)
                                continue

                            grd_obs.values *= para_ob1["multiple"]

                            time1_ = time1
                            if para_fo1["time_type"].lower() == "bt":
                                time1_ = time1 + datetime.timedelta(hours=8)
                            time1_ -= datetime.timedelta(hours=move_fo_time)


                            path1 = meteva.base.get_path(para_fo1["dirm"], time1_, dh+move_fo_time)

                            path1 = path1.replace("LLL", str(level))
                            grd_fo = para_fo1["read_method"](path1, grid=para["grid"], time=time1, dtime=dh,level = level,
                                                             data_name=model, show=True, **para_fo1["read_para"])
                            if grd_fo is not None:
                                grd_fo.values *= para_fo1["multiple"]
                                df_mid = meteva.perspact.middle_df_grd(grd_obs, grd_fo,mid_method, marker=marker,
                                                                       grade_list=para["grade_list"],compare=para["compare"])
                                mid_list.append(df_mid)
                            else:
                                print("faild to read " + path1)

                            # 当收集了超过500个时效的数据时就输出一次
                            if save_k % 500 == 0:
                                mid_all = meteva.base.concat(mid_list)
                                # 先删除文件在重新输出文件，避免文件大小膨胀
                                if os.path.exists(middle_result_path):
                                    os.remove(middle_result_path)
                                mid_all.to_hdf(middle_result_path, "df")  # 将结果输出到文件


            time1 = time1 + datetime.timedelta(hours=para["time_step"])
    mid_all = meteva.base.concat(mid_list)
    # 先删除文件在重新输出文件，避免文件大小膨胀
    if os.path.exists(middle_result_path):
        os.remove(middle_result_path)
    mid_all.to_hdf(middle_result_path, "df")  # 将结果输出到文件
    print("中间量统计程序运行完毕")
    print("中间结果已输出至"+middle_result_path)
    return





if __name__ == "__main__":


    pass


import pandas as pd
import numpy as np
import copy


def accumulation_strength_table(ob,fo, member_list = None,save_path=None):
    '''

    :param ob:
    :param fo:
    :param grade_list:
    :param save_path:
    :param member_list:
    :return:
    '''

    Fo_shape = fo.shape
    Ob_shape = ob.shape
    Ob_shpe_list = list(Ob_shape)
    size = len(Ob_shpe_list)
    ind = -size
    Fo_Ob_index = list(Fo_shape[ind:])

    if Fo_Ob_index != Ob_shpe_list:
        print('预报数据和观测数据维度不匹配')
        return
    Ob_shpe_list.insert(0, -1)
    new_Fo_shape = tuple(Ob_shpe_list)
    new_Fo = fo.reshape(new_Fo_shape)
    new_Fo_shape = new_Fo.shape
    if member_list is None:
        label = ["观测"]
        if new_Fo_shape[0] <= 1:
            label.append('预报')
        else:
            for i in range(new_Fo_shape[0]):
                label.append('预报' + str(i + 1))
    else:
        label = member_list

    max_ob = np.max(ob)
    max_fo = np.max(fo)
    max_value = max(max_ob,max_fo)+2
    grade_list = np.arange(1,max_value,1).tolist()

    index_list = ["(0," + str(grade_list[0])+")"]
    for index in range(len(grade_list) - 1):
        index_list.append("[" + str(grade_list[index]) + "," + str(grade_list[index + 1]) + ")")
    index_list.append(">=" + str(grade_list[-1]))
    conf_mx = np.zeros((len(label), len(index_list)))

    for line in range(new_Fo_shape[0]):
        fo_piece = new_Fo[line, :]
        if grade_list is not None:
            gle = [-1e300]
            gle.extend(grade_list)
            gle.append(1e300)
            for i in range(len(gle) - 1):
                if line == 0:
                    ob_index_list = np.where((ob >= gle[i]) & (ob < gle[i + 1]))
                    ob_part = ob[ob_index_list]
                    conf_mx[0, i] = np.sum(ob_part)
                fo_index_list = np.where((fo_piece >= gle[i]) & (fo_piece < gle[i + 1]))
                fo_part = fo_piece[fo_index_list]
                conf_mx[line + 1, i] = np.sum(fo_part)

    if save_path is not None:
        table_data = pd.DataFrame(conf_mx.T,
                                  index=pd.MultiIndex.from_product([['类别'], index_list]),
                                  columns=pd.MultiIndex.from_product([['ob-fo'], label])
                                  )
        table_data.to_excel(save_path, sheet_name='sheet1')
        print("累计降水量随强度变化表已以excel表格形式保存至" + save_path)
    return conf_mx

def accumulation_strenght_table(ob,fo, member_list = None,save_path=None):
   return accumulation_strength_table(ob,fo,member_list=member_list,save_path=save_path)


def frequency_strength_table(ob,fo, member_list = None,save_path=None):
    '''

    :param ob:
    :param fo:
    :param grade_list:
    :param save_path:
    :param member_list:
    :return:
    '''

    Fo_shape = fo.shape
    Ob_shape = ob.shape
    Ob_shpe_list = list(Ob_shape)
    size = len(Ob_shpe_list)
    ind = -size
    Fo_Ob_index = list(Fo_shape[ind:])

    if Fo_Ob_index != Ob_shpe_list:
        print('预报数据和观测数据维度不匹配')
        return
    Ob_shpe_list.insert(0, -1)
    new_Fo_shape = tuple(Ob_shpe_list)
    new_Fo = fo.reshape(new_Fo_shape)
    new_Fo_shape = new_Fo.shape

    if member_list is None:
        label = ["观测"]
        if new_Fo_shape[0] <= 1:
            label.append('预报')
        else:
            for i in range(new_Fo_shape[0]):
                label.append('预报' + str(i + 1))
    else:
        label = member_list

    max_ob = np.max(ob)
    max_fo = np.max(fo)
    max_value = max(max_ob,max_fo)+2
    grade_list = np.arange(1,max_value,1).tolist()

    index_list = ["(0," + str(grade_list[0])+")"]
    for index in range(len(grade_list) - 1):
        index_list.append("[" + str(grade_list[index]) + "," + str(grade_list[index + 1]) + ")")
    index_list.append(">=" + str(grade_list[-1]))
    conf_mx = np.zeros((len(label), len(index_list)))

    for line in range(new_Fo_shape[0]):
        fo_piece = new_Fo[line, :]
        if grade_list is not None:
            gle = [-1e300]
            gle.extend(grade_list)
            gle.append(1e300)
            for i in range(len(gle) - 1):
                if line == 0:
                    ob_index_list = np.where((ob >= gle[i]) & (ob < gle[i + 1]))
                    ob_part = ob[ob_index_list]
                    conf_mx[0, i] = ob_part.size
                fo_index_list = np.where((fo_piece >= gle[i]) & (fo_piece < gle[i + 1]))
                fo_part = fo_piece[fo_index_list]
                conf_mx[line + 1, i] = fo_part.size

    if save_path is not None:
        table_data = pd.DataFrame(conf_mx.T,
                                  index=pd.MultiIndex.from_product([['类别'], index_list]),
                                  columns=pd.MultiIndex.from_product([['ob-fo'], label])
                                  )
        table_data.to_excel(save_path, sheet_name='sheet1')
        print("降水频次随强度变化表已以excel表格形式保存至" + save_path)
    return conf_mx

def frequency_strenght_table(ob,fo, member_list = None,save_path=None):
    return frequency_strength_table(ob,fo,member_list=member_list,save_path = save_path)


# -*- coding: utf-8 -*-
"""
Created on Mon Mar  1 18:34:06 2021
"""
import numpy as np


def thresholder(x, th, func_type=None, rule=">=", replace_with=0, **args):
    if func_type is None:
        func_type = ['binary', 'replace_below']
    if rule == ">=":
        bool_value = x >= th
    elif rule == ">":
        bool_value = x > th
    elif rule == "<=":
        bool_value = x <= th
    elif rule == "<":
        bool_value = x < th
    xdim = x.shape
    if xdim is None:
        out = np.full(len(x), replace_with)
    else:
        out = np.full(xdim, replace_with)
    if func_type == "binary":
        out[bool_value] = 1
    else:
        out = x[bool_value]
    return out



# -*-coding:utf-8-*-
import numpy as np


def loc_list_setup(a, which_stats=None, nthresh=1, n_p=1, nk=1, nalpha=1):
    if which_stats is None:
        which_stats = ["bdelta", "haus", "qdmapdiff", "med", "msd", "ph", "fom", "minsep"]
    if a is not None:
        out = a
    else:
        out = {}
    q = nthresh
    outvec = []
    for i in range(q):
        outvec.append(None)
    if "bdelta" in which_stats:
        if q == 1:
            out['bdelta'] = np.full(n_p, None)
        else:
            out['bdelta'] = np.full((n_p, q), None)
    if "haus" in which_stats:
        out['haus'] = outvec
    if "qdmapdiff" in which_stats:
        if q == 1:
            out['qdmapdiff'] = np.full(nk, None)
        else:
            out['qdmapdiff'] = np.full((nk, q), None)
    if "med" in which_stats:
        out['medMiss'] = outvec
        out['medFalseAlarm'] = outvec
    if "msd" in which_stats:
        out['msdMiss'] = outvec
        out['msdFalseAlarm'] = outvec
    if "ph" in which_stats:
        if q == 1:
            out['ph'] = np.full(nk, None)
        else:
            out['ph'] = np.full((nk, q), None)
    if "fom" in which_stats:
        if q == 1:
            out['fom'] = np.full(nalpha, None)
        else:
            out['fom'] = np.full((nalpha, q), None)
    if "minsep" in which_stats:
        out['minsep'] = outvec
    return out



import numpy as np


def mij(x, s=None, i=0, j=0):
    x = np.array(x)
    if s is None:
        xdim = x.shape
        range0 = np.tile(np.arange(xdim[0]), xdim[1])
        range1 = (np.arange(xdim[1])).repeat(xdim[0])
        s = np.stack((range0, range1), axis=-1)
    s[:, 0] = np.power(s[:, 0], i)
    s[:, 1] = np.power(s[:, 1], j)
    s = np.prod(s, axis=1)
    result = np.sum(s * np.reshape(x, -1, 'F'))
    return result



# -*- coding: utf-8 -*-
"""
Created on Wed Jan  6 16:03:10 2021

"""
import copy
import meteva
import numpy as np
import matplotlib.pyplot as plt
import math
from matplotlib.colors import BoundaryNorm
import os
from .feature_match_analyzer import get_summary


def add_map(ax, add_county_line=False, add_worldmap=True, title=None, sup_fontsize=12):
    if add_worldmap:
        meteva.base.tool.plot_tools.add_china_map_2basemap(ax, name="world", edgecolor='k', lw=0.3, encoding='gbk',
                                                           grid0=None)  # "国界"
    meteva.base.tool.plot_tools.add_china_map_2basemap(ax, name="nation", edgecolor='k', lw=0.3,
                                                       encoding='gbk')  # "国界"
    meteva.base.tool.plot_tools.add_china_map_2basemap(ax, edgecolor='k', lw=0.3, encoding='gbk')  # "省界"
    if add_county_line:
        meteva.base.tool.plot_tools.add_china_map_2basemap(ax, name="county", edgecolor='k', lw=0.2,
                                                           encoding='gbk')  # "省界"
    ax.set_title(title, fontsize=sup_fontsize * 0.9)
    return ax


class mfig:

    def __init__(self,nplot,map_extend,ncol = None,height  = None,width = None,dpi = 300,sup_title = None,sup_fontsize = 12,
              add_county_line = False,add_worldmap = True,title_list = None):

        if isinstance(map_extend, list):
            slon = map_extend[0]
            elon = map_extend[1]
            slat = map_extend[2]
            elat = map_extend[3]
            rlon = elon - slon
            rlat = elat - slat
        else:
            slon = map_extend.slon
            slat = map_extend.slat
            elon = map_extend.elon
            elat = map_extend.elat
            rlon = map_extend.elon - map_extend.slon
            rlat = map_extend.elat - map_extend.slat

        if sup_title is None:
            sup_height_title = 0
        else:
            sup_height_title = sup_fontsize * 0.12

        height_title = sup_fontsize * 0.1
        height_bottem_xticsk = sup_fontsize * 0.05
        height_hspace = sup_fontsize * 0.04

        width_wspace = height_hspace * 1.5

        width_colorbar = 0.5
        width_left_yticks = sup_fontsize * 0.2

        if ncol is None:
            match_list = []
            for i in range(nplot, 0, -1):
                ncol = i
                nrow = int(math.ceil(nplot / ncol))
                rate = ncol * rlon / (nrow * rlat)
                if rate < 2 and rate > 9 / 16:
                    match_list.append([i, ncol * nrow - nplot])
            if len(match_list) == 0:
                ncol = nplot
            else:
                match_array = np.array(match_list)
                min_index = np.argmin(match_array[:, 1])
                ncol = match_array[min_index, 0]
        nrow = int(math.ceil(nplot / ncol))

        if width is None and height is None:
            width = 8

        if width is None:
            height_all_plot = height - height_title - height_bottem_xticsk - (
                        nrow - 1) * height_hspace + sup_height_title
            height_map = height_all_plot / nrow
            width_map = height_map * rlon / rlat
            width_all_plot = width_map * ncol + (ncol - 1) * width_wspace
            width = width_all_plot + width_colorbar + width_left_yticks
        else:
            width_all_plot = width - width_colorbar - width_left_yticks - (ncol - 1) * width_wspace
            width_map = width_all_plot / ncol
            height_map = width_map * rlat / rlon
            height_all_plot = height_map * nrow + (nrow - 1) * height_hspace
            height = height_all_plot + height_title + height_bottem_xticsk + sup_height_title

        vmax = elon
        vmin = slon
        r = rlon
        if r <= 1:
            inte = 0.1
        elif r <= 5 and r > 1:
            inte = 1
        elif r <= 10 and r > 5:
            inte = 2
        elif r < 20 and r >= 10:
            inte = 4
        elif r <= 30 and r >= 20:
            inte = 5
        elif r < 180:
            inte = 10
        else:
            inte = 20

        vmin = inte * (math.ceil(vmin / inte))
        vmax = inte * ((int)(vmax / inte) + 1)

        xticks = np.arange(vmin, vmax, inte)
        xticks_label = []
        xticks_label_None = []
        for x in range(len(xticks)):
            xticks_label.append(str(round(xticks[x], 6)))
            xticks_label_None.append("")
        if xticks[-1] > 0:
            xticks_label[-1] = "   " + xticks_label[-1] + "°E"
        else:
            xticks_label[-1] = "   " + xticks_label[-1] + "°W"

        vmax = elat
        vmin = slat
        r = rlat
        if r <= 1:
            inte = 0.1
        elif r <= 5 and r > 1:
            inte = 1
        elif r <= 10 and r > 5:
            inte = 2
        elif r < 20 and r >= 10:
            inte = 4
        elif r <= 30 and r >= 20:
            inte = 5
        else:
            inte = 10

        vmin = inte * (math.ceil(vmin / inte))
        vmax = inte * ((int)(vmax / inte) + 1)
        yticks = np.arange(vmin, vmax, inte)
        yticks_label = []
        yticks_label_None = []
        for y in range(len(yticks)):
            if yticks[y] >= 0:
                yticks_label.append(str(round(yticks[y], 6)) + "°N")
            else:
                yticks_label.append(str(round(-yticks[y], 6)) + "°S")
            yticks_label_None.append("")

        fig = plt.figure(figsize=(width, height), dpi=dpi)
        y_sup_title = (height_bottem_xticsk - 0.1 + (nrow) * (height_map + height_hspace)) / height
        if sup_title is not None:
            plt.suptitle(sup_title, x=0.6, y=y_sup_title, fontsize=sup_fontsize)
        ax_list = []
        for p in range(nplot):
            pi = p % ncol
            pj = int(p / ncol)

            rect1 = [(width_left_yticks + pi * (width_map + width_wspace)) / width,
                     (height_bottem_xticsk + (nrow - 1 - pj) * (height_map + height_hspace)) / height,
                     width_map / width,
                     height_map / height]
            ax = plt.axes(rect1)
            ax.set_xlim((slon, elon))
            ax.set_ylim((slat, elat))

            vmax = elon
            vmin = slon
            r = rlon
            if r <= 1:
                inte = 0.1
            elif r <= 5 and r > 1:
                inte = 1
            elif r <= 10 and r > 5:
                inte = 2
            elif r < 20 and r >= 10:
                inte = 4
            elif r <= 30 and r >= 20:
                inte = 5
            elif r < 180:
                inte = 10
            else:
                inte = 20

            vmin = inte * (math.ceil(vmin / inte))
            vmax = inte * ((int)(vmax / inte) + 1)
            xticks = np.arange(vmin, vmax, inte)
            xticks_label = []
            for x in range(len(xticks)):
                xticks_label.append(str(round(xticks[x], 6)))
            if xticks[-1] > 0:
                xticks_label[-1] = "   " + xticks_label[-1] + "°E"
            else:
                xticks_label[-1] = "   " + xticks_label[-1] + "°W"
            ax.set_xticks(xticks)
            ax.set_xticklabels(xticks_label, fontsize=sup_fontsize * 0.9) #, family='Times New Roman')

            vmax = elat
            vmin = slat
            r = rlat
            if r <= 1:
                inte = 0.1
            elif r <= 5 and r > 1:
                inte = 1
            elif r <= 10 and r > 5:
                inte = 2
            elif r < 20 and r >= 10:
                inte = 4
            elif r <= 30 and r >= 20:
                inte = 5
            else:
                inte = 10

            vmin = inte * (math.ceil(vmin / inte))
            vmax = inte * ((int)(vmax / inte) + 1)
            yticks = np.arange(vmin, vmax, inte)
            yticks_label = []
            for y in range(len(yticks)):

                if yticks[y] >= 0 and y == len(yticks) - 1:
                    yticks_label.append(str(round(yticks[y], 6)) + "°N")
                elif yticks[y] < 0 and y == 0:
                    yticks_label.append(str(round(-yticks[y], 6)) + "°S")
                elif yticks[y] == 0:
                    yticks_label.append("EQ" + "  ")
                else:
                    yticks_label.append(str(round(yticks[y], 6)) + "    ")
            ax.set_yticks(yticks)
            ax.set_yticklabels(yticks_label, fontsize=sup_fontsize * 0.9) #, family='Times New Roman')
            if title_list is None:
                sub_title = None
            else:
                sub_title = title_list[p]
            add_map(ax, add_county_line=add_county_line, add_worldmap=add_worldmap, sup_fontsize=sup_fontsize,
                    title=sub_title)
            ax_list.append(ax)

        self.ax_list = ax_list
        self.sup_fontsize = sup_fontsize
        self.ncol = ncol


    def add_contourf(self,i,grd,cmap ="rainbow",clevs= None,add_colorbar = True):
        ax = self.ax_list[i]
        x = grd['lon'].values
        y = grd['lat'].values
        vmax = np.max(grd.values)
        vmin = np.min(grd.values)
        cmap1,clevs1 = meteva.base.tool.color_tools.def_cmap_clevs(cmap=cmap,clevs=clevs,vmin=vmin,vmax = vmax)
        norm = BoundaryNorm(clevs1, ncolors=cmap1.N-1)
        im = ax.contourf(x, y, np.squeeze(grd.values), levels=clevs1, cmap=cmap1,norm = norm)

        time_str = meteva.base.tool.time_tools.time_to_str(grd["time"].values[0])
        dati_str = time_str[0:4] + "年" + time_str[4:6] + "月" + time_str[6:8] + "日" + time_str[8:10] + "时"
        title1 = str(grd["member"].values[0]) + " " + dati_str + str(grd["dtime"].values[0]) + "H时效 "
        if "var_name" in grd.attrs.keys():
            title1 = title1 + grd.attrs["var_name"]
        ax.set_title(title1, fontsize=self.sup_fontsize* 0.9, pad=0)

        fig = plt.gcf()
        width = fig.bbox.width/fig.dpi
        height = fig.bbox.height/fig.dpi
        location = [ax.bbox.x1/fig.dpi/width+0.005, ax.bbox.y0 / fig.dpi/height, 0.01, ax.bbox.height/fig.dpi/height]

        if(add_colorbar):
            colorbar_position = fig.add_axes(location)  # 位置[左,下,宽,高]
            cb = plt.colorbar(im,cax= colorbar_position)
            cb.ax.tick_params(labelsize=self.sup_fontsize * 0.8)

    def add_mesh(self,i,grd,cmap = "rainbow",clevs = None,add_colorbar = True,matched = True,title = None):
        ax = self.ax_list[i]
        x = grd['lon'].values
        y = grd['lat'].values
        vmax = np.max(grd.values)
        vmin = np.min(grd.values)
        cmap1,clevs1 = meteva.base.tool.color_tools.def_cmap_clevs(cmap=cmap,clevs=clevs,vmin=vmin,vmax = vmax)
        norm = BoundaryNorm(clevs1, ncolors=cmap1.N-1)
        #im = ax.pcolormesh(x, y, np.squeeze(grd.values), levels=clevs1, cmap=cmap1,norm = norm)
        im = ax.pcolormesh(x, y, np.squeeze(grd.values), cmap=cmap1, norm=norm)
        fig = plt.gcf()
        width = fig.bbox.width/fig.dpi
        height = fig.bbox.height/fig.dpi
        location = [ax.bbox.x1/fig.dpi/width+0.005, ax.bbox.y0 / fig.dpi/height, 0.01, ax.bbox.height/fig.dpi/height]

        if(add_colorbar):
            colorbar_position = fig.add_axes(location)  # 位置[左,下,宽,高]
            cb =  plt.colorbar(im,cax= colorbar_position)
            if matched :
                cticks = [-1]
                cticks.extend((np.array(clevs1[2:])+0.5).tolist())
                cticks = np.array(cticks)
                cb.set_ticks(cticks)
                clabels = ["未\n配\n对"]
                for i in range(2,len(clevs1)):
                    clabels.append(str(int(clevs1[i]+0.5)))
                    #clabels.extend((np.array(clevs1[2:])+0.5).tolist())
                cb.set_ticklabels(clabels)
            else:
                cticks=(np.array(clevs1[2:])+0.5).tolist()
                cticks = np.array(cticks)
                cb.set_ticks(cticks)
                clabels = []
                #clabels = ["未\n配\n队"]
                for i in range(2,len(clevs1)):
                    clabels.append(str(int(clevs1[i]+0.5)))
                cb.set_ticklabels(clabels)
            cb.ax.tick_params(labelsize=self.sup_fontsize * 0.8)

        if title is not None:
            title1 = title
        else:
            time_str = meteva.base.tool.time_tools.time_to_str(grd["time"].values[0])
            dati_str = time_str[0:4] + "年" + time_str[4:6] + "月" + time_str[6:8] + "日" + time_str[8:10] + "时"
            title1 = str(grd["member"].values[0]) + " " + dati_str + str(grd["dtime"].values[0]) + "H时效 "
            if "var_name" in grd.attrs.keys():
                title1 = title1 + grd.attrs["var_name"]
        ax.set_title(title1, fontsize=self.sup_fontsize* 0.9, pad=0)



def plot_value_list(look_list, cmap ="rain_24h",clevs = None,save_path = None,show = False,sup_fontsize=10,dpi = 300,mfg1 = None):

    ncol = len(look_list)
    grid1 = look_list[0]["grid"]
    # grd_ob = look["grd_ob"]
    # grd_fo = look["grd_fo"]
    # grd_ob_smooth = look["grd_ob_smooth"]
    # grd_fo_smooth = look["grd_fo_smooth"]
    # vmax = np.max((np.max(grd_ob), np.max(grd_fo)))
    cmap1, clevs1 = meteva.base.tool.color_tools.def_cmap_clevs(cmap=cmap, clevs=clevs)

    mfg1_show = True
    if mfg1 is None:
        mfg1 = mfig(2*ncol,map_extend=grid1,ncol=ncol,sup_fontsize=sup_fontsize,dpi=dpi)
    else:
        mfg1_show = False

    for i in range(ncol):
        look1 = look_list[i]
        grd = look1["grd"]
        grd_smooth = look1["grd_smooth"]
        mfg1.add_contourf(i,grd,cmap = cmap1,clevs=clevs1,add_colorbar=(i==ncol-1))
        mfg1.add_contourf(ncol + i, grd_smooth, cmap=cmap1, clevs=clevs1,add_colorbar=(i==ncol-1))

    if mfg1_show:
        if save_path is None:
            show = True
        if save_path is not None:
            meteva.base.tool.path_tools.creat_path(save_path)
            file1, extension = os.path.splitext(save_path)
            if(len(extension) ==0):
                print("save_path中没包含后缀，如.png等,未能输出至指定路径")
                return
            extension = extension[1:]
            plt.savefig(save_path,format = extension,bbox_inches='tight')
            print("图片已保存至" + save_path)
        if show:
            plt.show()
        plt.close()




def plot_value_and_label_list(look_list,cmap = "rain_24h",clevs = None,save_path = None,sup_fontsize=10,dpi = 300,show = False):
    ncol = len(look_list)
    grid1 = look_list[0]["grid"]
    mfg1 = mfig(3*ncol, map_extend=grid1, ncol=ncol, sup_fontsize=sup_fontsize, dpi=dpi)
    plot_value_list(look_list, cmap, clevs, mfg1=mfg1)
    plot_label_list(look_list, mfg1=mfg1)
    if save_path is None:
        show = True

    if save_path is not None:
        meteva.base.tool.path_tools.creat_path(save_path)
        file1, extension = os.path.splitext(save_path)
        if (len(extension) == 0):
            print("save_path中没包含后缀，如.png等,未能输出至指定路径")
            return
        extension = extension[1:]
        plt.savefig(save_path, format=extension, bbox_inches='tight')
        print("图片已保存至" + save_path)
    if show:
        plt.show()
    plt.close()

def plot_value_and_label(look,cmap = "rain_24h",clevs = None,save_path = None,sup_fontsize=10,dpi = 300,show = False):

    if isinstance(look,list):
        plot_value_and_label_list(look,cmap =cmap,clevs = clevs,save_path = save_path,sup_fontsize=sup_fontsize,dpi = dpi,show = show)
    else:
        grid1 = look["grid"]
        mfg1 = mfig(6,map_extend=grid1,ncol=2,sup_fontsize=sup_fontsize,dpi=dpi)
        plot_value(look,cmap,clevs,mfg1 = mfg1)
        plot_label(look,mfg1 = mfg1)
        if save_path is None:
            show = True

        if save_path is not None:
            meteva.base.tool.path_tools.creat_path(save_path)
            file1, extension = os.path.splitext(save_path)
            if(len(extension) ==0):
                print("save_path中没包含后缀，如.png等,未能输出至指定路径")
                return
            extension = extension[1:]
            plt.savefig(save_path,format = extension,bbox_inches='tight')
            print("图片已保存至" + save_path)
        if show:
            plt.show()
        plt.close()


def plot_value(look, cmap ="rain_24h",clevs = None,save_path = None,show = False,sup_fontsize=10,dpi = 300,mfg1 = None):
    grid1 = look["grid"]
    grd_ob = look["grd_ob"]
    grd_fo = look["grd_fo"]
    grd_ob_smooth = look["grd_ob_smooth"]
    grd_fo_smooth = look["grd_fo_smooth"]
    vmax = np.max((np.max(grd_ob), np.max(grd_fo)))
    cmap1, clevs1 = meteva.base.tool.color_tools.def_cmap_clevs(cmap=cmap, clevs=clevs, vmin=0, vmax=vmax)

    mfg1_show = True
    if mfg1 is None:
        mfg1 = mfig(4,map_extend=grid1,ncol=2,sup_fontsize=sup_fontsize,dpi=dpi)
    else:
        mfg1_show = False

    mfg1.add_contourf(0,grd_ob,cmap = cmap1,clevs=clevs1,add_colorbar=False)
    mfg1.add_contourf(1, grd_fo, cmap=cmap1, clevs=clevs1)
    mfg1.add_contourf(2, grd_ob_smooth, cmap=cmap1, clevs=clevs1,add_colorbar=False)
    mfg1.add_contourf(3, grd_fo_smooth, cmap=cmap1, clevs=clevs1)

    if mfg1_show:
        if save_path is None:
            show = True
        if save_path is not None:
            meteva.base.tool.path_tools.creat_path(save_path)
            file1, extension = os.path.splitext(save_path)
            if(len(extension) ==0):
                print("save_path中没包含后缀，如.png等,未能输出至指定路径")
                return
            extension = extension[1:]
            plt.savefig(save_path,format = extension,bbox_inches='tight')
            print("图片已保存至" + save_path)
        if show:
            plt.show()
        plt.close()


def add_pts(ax,map_extend,line_dict,nmatched,line_width = None,sup_fontsize=10,dpi = 300):
    if isinstance(map_extend,list):
        slon = map_extend[0]
        elon = map_extend[1]
        slat = map_extend[2]
        elat = map_extend[3]

    elif isinstance(map_extend,meteva.base.grid):
        slon = map_extend.slon
        slat = map_extend.slat
        elon = map_extend.elon
        elat = map_extend.elat

    rlon = elon - slon
    rlat = elat - slat
    fig = plt.gcf()
    map_width = ax.bbox.width / fig.dpi

    if line_width is None:
        line_width = map_width *  0.2

    for key in line_dict.keys():
        if key <= nmatched:
            c = "r"
        else:
            c = "k"
        point0 = line_dict[key]
        point = np.zeros([point0.shape[0]+1,point0.shape[1]])
        point[:-1,:] = point0[:,:]
        point[-1,:] = point0[0,:]
        ax.plot(point[:, 0], point[:, 1], "k", linewidth=line_width)
        max_j = np.argmax(point[:,1])
        dlat = 0.1 * rlon / map_width
        lat = point[max_j,1] + dlat * 0.2
        lat1 = point[max_j,1] + dlat * 1.2
        str1 = str(key)
        dlon = 0.1 * len(str1) * rlon / map_width
        if lat1 > elat:
            min_j = np.argmin(point[:,1])
            lon = point[min_j,0]
            lon = min(lon,elon - dlon)
            lon = max(lon,slon + dlon)
            lat = point[min_j,1] - dlat * 0.2
            ax.text(lon,lat,str(key),c = c,horizontalalignment='center',fontweight='bold',
                    verticalalignment = "top")
        else:
            lon = point[max_j,0]
            lon = min(lon,elon - dlon)
            lon = max(lon,slon + dlon)
            ax.text(lon,lat,str(key),c = c,horizontalalignment='center',fontweight='bold')



    ax.set_xlim(slon,elon)
    ax.set_ylim(slat,elat)



def plot_label_list(look_list,ncol = None,save_path = None,show = False,sup_fontsize=10,dpi = 300,mfg1 = None,
                    title_list = None):

    nlook = len(look_list)
    grid1 = look_list[0]["grid"]
    start_ax = 2 * nlook
    mfg1_show = True

    if mfg1 is None:
        mfg1 = mfig(nlook,map_extend=grid1,ncol=ncol,sup_fontsize=sup_fontsize,dpi=dpi)
        start_ax = 0
        ncol = mfg1.ncol
    else:
        mfg1_show = False
        ncol = nlook



    vmax = 1
    label_list = []
    for i in range(nlook):
        look =look_list[i]
        nmatch = 1000
        if "match_count" in look.keys():
            nmatch = look["match_count"]
        data_labeled = look['grd_label'].copy()
        data_labeled.values[data_labeled.values>nmatch] = -1
        label_list.append(data_labeled)
        vmax1 = np.max(data_labeled)
        if vmax1 > vmax:
            vmax = vmax1

    cmap2, clevs2 = meteva.base.tool.color_tools.def_cmap_clevs(cmap="mode", clevs=None, vmin=0, vmax=vmax)

    for i in range(nlook):
        look = look_list[i]
        nmatch = 1000
        if "match_count" in look.keys():
            matched = True
            nmatch = look["match_count"]
        else:
            matched = False
        title1 = None
        if title_list is not None:
            title1= title_list[i]
        mfg1.add_mesh(start_ax+i,label_list[i],cmap=cmap2,clevs=clevs2,add_colorbar=((i+1)%ncol==0),matched = matched,
                      title = title1)

        label_count = look["grd_features"]["label_count"]
        id_list = look["id_list"]
        pts_dict = {}
        for j in range(len(id_list)):
            id1 = id_list[j]
            feature = meteva.method.feature_axis(look,id1, None)
            if feature is not None:
                pts = feature["pts"]
                pts_dict[id1] =pts
        add_pts(mfg1.ax_list[start_ax + i],look["grid"],pts_dict,nmatch)

    if mfg1_show:
        if save_path is None:
            show = True
        if save_path is not None:
            meteva.base.tool.path_tools.creat_path(save_path)
            file1, extension = os.path.splitext(save_path)
            if(len(extension) ==0):
                print("save_path中没包含后缀，如.png等,未能输出至指定路径")
                return
            extension = extension[1:]
            plt.savefig(save_path,format = extension,bbox_inches='tight')
            print("图片已保存至" + save_path)
        if show:
            plt.show()
        plt.close()

def plot_label(look,save_path = None,show = False,sup_fontsize=10,dpi = 300,mfg1 = None):

    grid1 = look["grid"]
    start_ax = 4
    mfg1_show = True

    if mfg1 is None:
        mfg1 = mfig(2,map_extend=grid1,ncol=2,sup_fontsize=sup_fontsize,dpi=dpi)
        start_ax = 0
    else:
        mfg1_show = False

    nmatch = 1000
    matched = False
    data_Xlabeled = look['grd_ob_label'].copy()
    data_Ylabeled = look['grd_fo_label'].copy()
    if "match_count" in look.keys():
        nmatch = 0
        if len(look["label_list_matched"])>0:
            nmatch = np.max(np.array(look["label_list_matched"]))
        matched = True

        unmatched = look["unmatched"]["ob"]
        for id in unmatched:
            data_Xlabeled.values[data_Xlabeled.values ==id] = -1
        unmatched = look["unmatched"]["fo"]
        for id in unmatched:
            data_Ylabeled.values[data_Ylabeled.values ==id] = -1

    vmax = np.max((np.max(data_Xlabeled), np.max(data_Ylabeled))) + 1
    cmap2, clevs2 = meteva.base.tool.color_tools.def_cmap_clevs(cmap="mode", clevs=None, vmin=0, vmax=vmax)

    mfg1.add_mesh(start_ax,data_Xlabeled,cmap=cmap2,clevs=clevs2,add_colorbar=False,matched = matched)
    mfg1.add_mesh(start_ax+1, data_Ylabeled, cmap=cmap2, clevs=clevs2,add_colorbar=True,matched = matched)

    #ax_list = plot_2d_grid_list([data_Xlabeled, data_Ylabeled],type="mesh",
    #            cmap = "mode", vmax = np.max((np.max(data_Xlabeled), np.max(data_Ylabeled)))+1,ncol=2)
    nob = look["grd_ob_features"]["label_count"]

    pts_dict = {}
    for i in range(nob):
        feature = meteva.method.feature_axis(look, i + 1, "ob")
        pts = feature["pts"]
        pts_dict[i+1] =pts
    add_pts(mfg1.ax_list[start_ax+0],look["grid"],pts_dict,nmatch)

    #nfo = look["grd_fo_features"]["label_count"]
    label_list_fo = look["label_list_fo"]
    pts_dict = {}
    for id in label_list_fo:
        feature = meteva.method.feature_axis(look, id, "fo")
        pts = feature["pts"]
        pts_dict[id] = pts
    add_pts(mfg1.ax_list[start_ax+1],look["grid"],pts_dict,nmatch)
    if mfg1_show:
        if save_path is None:
            show = True
        if save_path is not None:
            meteva.base.tool.path_tools.creat_path(save_path)
            file1, extension = os.path.splitext(save_path)
            if(len(extension) ==0):
                print("save_path中没包含后缀，如.png等,未能输出至指定路径")
                return
            extension = extension[1:]
            plt.savefig(save_path,format = extension,bbox_inches='tight')
            print("图片已保存至" + save_path)
        if show:
            plt.show()
        plt.close()


def plot_interest(interest,save_path = None,show = False):
    shape = list(interest["interest"].shape)
    shape[2] = shape[2]+1
    dat = np.zeros(shape)
    dat[:,:,:-1] = interest["interest"][:,:,:]
    dat[:,:,-1] = interest["total_interest"][:,:]
    p_list = copy.deepcopy(interest["properties"])
    p_list.append("total interest")
    name_list_dict = {
        "fo_label_id": interest["label_list_fo"],
        "ob_label_id": interest["label_list_ob"],
        "properties":p_list
    }
    meteva.base.tool.plot_tools.mesh(dat, name_list_dict=name_list_dict, annot=2, axis_x="ob_label_id",
                             axis_y="fo_label_id",save_path = save_path,show = show,ncol = 2)

def plot_feature(feature,save_path = None,show = False,dpi = 100):

    feature = get_summary(feature)
    sup_fontsize = 10
    row = 43
    col1 = 10
    label_list_matched = feature["label_list_matched"]
    nmatch = len(label_list_matched)
    col2 = 6 + nmatch
    col = max(col1,col2)

    width = col * sup_fontsize * 0.08
    height = row * sup_fontsize * 0.02
    fig = plt.figure(figsize=(width,height),dpi =dpi)
    axis = plt.axes([0,0,1,1])

    axis.axes.set_axis_off()
    axis.set_xlim(0,col)
    axis.set_ylim(0,row)
    plt.hlines(0.05, 0, col, "k", linewidth=0.5)
    for i in range(1,14):
        plt.hlines(i,2,col,"k",linewidth = 0.5,linestyles="--")
    plt.hlines(14,0,col,"k",linewidth = 0.5)

    for i in range(14,22):
        plt.hlines(i,2  + 2*(i%2),col,"k",linewidth = 0.5,linestyles="--")
    plt.hlines(22,0,col,"k",linewidth = 0.5)

    for i in range(22,36):
        plt.hlines(i,2  + 2*(i%2),col,"k",linewidth = 0.5,linestyles="--")
    plt.hlines(36,0,col,"k",linewidth = 0.5)
    plt.hlines(37,0,col,"k",linewidth = 0.5)
    plt.hlines(38,0,col,"k",linewidth = 1,linestyles="-")
    plt.hlines(39,0,col,"k",linewidth = 0.5,linestyles="--")
    plt.hlines(40,2,col,"k",linewidth = 0.5,linestyles="-")
    plt.hlines(41, 2, col, "k", linewidth=0.5, linestyles="--")
    plt.hlines(42,0,col,"k",linewidth = 1,linestyles="-")
    row1 = row-1+0.2
    plt.text(0,row1,"起报时间:"+feature["time"],fontsize=sup_fontsize)
    plt.text(3,row1,"预报时效:"+str(feature["dtime"]),fontsize=sup_fontsize)
    plt.text(6,row1,"成功匹配目标数:"+str(nmatch),fontsize=sup_fontsize)
    plt.text(0,row1-1,"整场目标预报评价",fontsize=sup_fontsize)
    plt.text(0,row1-1,"整场目标预报评价",fontsize=sup_fontsize)

    keys = ["Hits","Misses","False alarms","Correct negatives"]
    values = feature["feature_table"]["contingency_table_yesorno"]

    for i in range(len(keys)):
        plt.text(i*2+2,row1-1,keys[i],fontsize=sup_fontsize)
        plt.text(i*2+2,row1-2, int(values[keys[i]]), fontsize=sup_fontsize)


    keys = ["ets","pod","pofd","far","hss"]
    values = feature["feature_table"]["score"]
    for i in range(len(keys)):
        plt.text(i*2,row1-3,keys[i],fontsize=sup_fontsize)
        plt.text(i*2, row1-4, "{:.3f}".format(values[keys[i]]), fontsize=sup_fontsize)


    plt.text(0, row1-5, "逐个目标属性检验", fontsize=sup_fontsize)

    for i in range(nmatch):
        id = label_list_matched[i]
        plt.text(6+i, row1-5, "目标"+str(id), fontsize=sup_fontsize)

    plt.text(0, row1-6, "目标整体相似度", fontsize=sup_fontsize)
    for i in range(nmatch):
        plt.text(6+i, row1-6, "{:.3f}".format(feature["interester"][i]), fontsize=sup_fontsize)

    plt.text(0, row1-7, "目标轴属性", fontsize=sup_fontsize)

    names = ["主轴长度","次轴长度","主轴倾角","矩形窗_x0","矩形窗_y0","矩形窗_x1","矩形窗_y1","质心_x","质心_y","面积","中位数强度"]
    for i in range(len(names)):
        plt.text(2, row1-(7+i*2), names[i], fontsize=sup_fontsize)
        plt.text(4, row1-(7+i*2), "观测", fontsize=sup_fontsize)
        plt.text(4, row1-(8+i*2), "预报", fontsize=sup_fontsize)


    for i in range(nmatch):
        keys1 = ["ob","fo"]
        for j in range(2):
            id = label_list_matched[i]
            values = feature[id]["feature_axis"][keys1[j]]

            plt.text(6+i, row1-(7+j), "{:.3f}".format(values["lengths"]["MajorAxis"]), fontsize=sup_fontsize)
            plt.text(6+i, row1-(9+j), "{:.3f}".format(values["lengths"]["MinorAxis"]), fontsize=sup_fontsize)
            plt.text(6+i, row1-(11+j), "{:.3f}".format(values["OrientationAngle"]["MajorAxis"]), fontsize=sup_fontsize)
            plt.text(6+i, row1-(13+j), "{:.3f}".format(values["window"]["x0"]), fontsize=sup_fontsize)
            plt.text(6+i, row1-(15+j), "{:.3f}".format(values["window"]["y0"]), fontsize=sup_fontsize)
            plt.text(6+i, row1-(17+j), "{:.3f}".format(values["window"]["x1"]), fontsize=sup_fontsize)
            plt.text(6+i, row1-(19+j), "{:.3f}".format(values["window"]["y1"]), fontsize=sup_fontsize)

            values = feature[id]["feature_props"][keys1[j]]
            plt.text(6+i, row1-(21+j), "{:.3f}".format(values["centroid"]["x"]), fontsize=sup_fontsize)
            plt.text(6+i, row1-(23+j), "{:.3f}".format(values["centroid"]["y"]), fontsize=sup_fontsize)
            plt.text(6+i, row1-(25+j), "{:.3f}".format(values["area"]), fontsize=sup_fontsize)
            plt.text(6+i, row1-(27+j), "{:.3f}".format(values["intensity"][4]), fontsize=sup_fontsize)



    names = ["质心距离","角度差","面积比","重叠面积比例","bearing方位角","bdelta距离","haus距离","medMiss","medFalseAlarm","msdMiss",
             "msdFalseAlarm","ph距离","fom","minsep最近距离"]
    keys = ["cent_dist","angle_diff","area_ratio","int_area","bearing",
            "bdelta","haus","medMiss","medFalseAlarm","msdMiss","msdFalseAlarm","ph","fom","minsep"]

    for i in range(len(names)):
        plt.text(2, row1 - (29+i), names[i], fontsize=sup_fontsize)

        for j in range(nmatch):
            id = label_list_matched[j]
            plt.text(6+j,row1-(29+i), "{:.3f}".format(feature[id]["feature_comps"][keys[i]]), fontsize=sup_fontsize)

    plt.text(0, row1-21, "目标面属性", fontsize=sup_fontsize)
    plt.text(0, row1-29, "目标属性对比", fontsize=sup_fontsize)
    if save_path is None:
        show = True

    if save_path is not None:
        meteva.base.tool.path_tools.creat_path(save_path)
        file1, extension = os.path.splitext(save_path)
        if (len(extension) == 0):
            print("save_path中没包含后缀，如.png等,未能输出至指定路径")
            return
        extension = extension[1:]
        plt.savefig(save_path, format=extension, bbox_inches='tight')
        print("图片已保存至" + save_path)
    if show:
        plt.show()
    plt.close()




